"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def put_multiple(
    hub, ctx, rule: str, targets: List, event_bus_name: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Adds the specified targets to the specified rule, or updates the targets if they are already associated with the rule. Targets are the resources that are invoked when a rule is triggered. You can configure the following as targets for Events:    API destination    Amazon API Gateway REST API endpoints   API Gateway   AWS Batch job queue   CloudWatch Logs group   CodeBuild project   CodePineline   Amazon EC2 CreateSnapshot API call   Amazon EC2 RebootInstances API call   Amazon EC2 StopInstances API call   Amazon EC2 TerminateInstances API call   Amazon ECS tasks   Event bus in a different AWS account or Region. You can use an event bus in the US East (N. Virginia) us-east-1, US West (Oregon) us-west-2, or Europe (Ireland) eu-west-1 Regions as a target for a rule.   Firehose delivery stream (Kinesis Data Firehose)   Inspector assessment template (Amazon Inspector)   Kinesis stream (Kinesis Data Stream)   AWS Lambda function   Redshift clusters (Data API statement execution)   Amazon SNS topic   Amazon SQS queues (includes FIFO queues   SSM Automation   SSM OpsItem   SSM Run Command   Step Functions state machines   Creating rules with built-in targets is supported only in the AWS Management Console. The built-in targets are EC2 CreateSnapshot API call, EC2 RebootInstances API call, EC2 StopInstances API call, and EC2 TerminateInstances API call.  For some target types, PutTargets provides target-specific parameters. If the target is a Kinesis data stream, you can optionally specify which shard the event goes to by using the KinesisParameters argument. To invoke a command on multiple EC2 instances with one rule, you can use the RunCommandParameters field. To be able to make API calls against the resources that you own, Amazon EventBridge (CloudWatch Events) needs the appropriate permissions. For AWS Lambda and Amazon SNS resources, EventBridge relies on resource-based policies. For EC2 instances, Kinesis data streams, AWS Step Functions state machines and API Gateway REST APIs, EventBridge relies on IAM roles that you specify in the RoleARN argument in PutTargets. For more information, see Authentication and Access Control in the Amazon EventBridge User Guide. If another AWS account is in the same region and has granted you permission (using PutPermission), you can send events to that account. Set that account's event bus as a target of the rules in your account. To send the matched events to the other account, specify that account's event bus as the Arn value when you run PutTargets. If your account sends events to another account, your account is charged for each sent event. Each event sent to another account is charged as a custom event. The account receiving the event is not charged. For more information, see Amazon EventBridge (CloudWatch Events) Pricing.   Input, InputPath, and InputTransformer are not available with PutTarget if the target is an event bus of a different AWS account.  If you are setting the event bus of another account as the target, and that account granted permission to your account through an organization instead of directly by the account ID, then you must specify a RoleArn with proper permissions in the Target structure. For more information, see Sending and Receiving Events Between AWS Accounts in the Amazon EventBridge User Guide. For more information about enabling cross-account events, see PutPermission.  Input, InputPath, and InputTransformer are mutually exclusive and optional parameters of a target. When a rule is triggered due to a matched event:   If none of the following arguments are specified for a target, then the entire event is passed to the target in JSON format (unless the target is Amazon EC2 Run Command or Amazon ECS task, in which case nothing from the event is passed to the target).   If Input is specified in the form of valid JSON, then the matched event is overridden with this constant.   If InputPath is specified in the form of JSONPath (for example, $.detail), then only the part of the event specified in the path is passed to the target (for example, only the detail part of the event is passed).   If InputTransformer is specified, then one or more specified JSONPaths are extracted from the event and used as values in a template that you specify as the input to the target.   When you specify InputPath or InputTransformer, you must use JSON dot notation, not bracket notation. When you add targets to a rule and the associated rule triggers soon after, new or updated targets might not be immediately invoked. Allow a short period of time for changes to take effect. This action can partially fail if too many requests are made at the same time. If that happens, FailedEntryCount is non-zero in the response and each entry in FailedEntries provides the ID of the failed target and the error code.

    Args:
        rule(str): The name of the rule.
        event_bus_name(str, optional): The name or ARN of the event bus associated with the rule. If you omit this, the default event bus is used. Defaults to None.
        targets(List): The targets to update or add to the rule.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.event.target.init.put_multiple(ctx, rule=value, targets=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.event.target.init.put_multiple rule=value, targets=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="events",
        operation="put_targets",
        op_kwargs={"Rule": rule, "EventBusName": event_bus_name, "Targets": targets},
    )


async def remove(
    hub, ctx, rule: str, ids: List, event_bus_name: str = None, force: bool = None
) -> Dict:
    r"""
    **Autogenerated function**

    Removes the specified targets from the specified rule. When the rule is triggered, those targets are no longer be invoked. When you remove a target, when the associated rule triggers, removed targets might continue to be invoked. Allow a short period of time for changes to take effect. This action can partially fail if too many requests are made at the same time. If that happens, FailedEntryCount is non-zero in the response and each entry in FailedEntries provides the ID of the failed target and the error code.

    Args:
        rule(str): The name of the rule.
        event_bus_name(str, optional): The name or ARN of the event bus associated with the rule. If you omit this, the default event bus is used. Defaults to None.
        ids(List): The IDs of the targets to remove from the rule.
        force(bool, optional): If this is a managed rule, created by an AWS service on your behalf, you must specify Force as True to remove targets. This parameter is ignored for rules that are not managed rules. You can check whether a rule is a managed rule by using DescribeRule or ListRules and checking the ManagedBy field of the response. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.event.target.init.remove(ctx, rule=value, ids=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.event.target.init.remove rule=value, ids=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="events",
        operation="remove_targets",
        op_kwargs={
            "Rule": rule,
            "EventBusName": event_bus_name,
            "Ids": ids,
            "Force": force,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def back(hub, ctx, anomaly_detector_arn: str) -> Dict:
    r"""
    **Autogenerated function**

    Runs a backtest for anomaly detection for the specified resource.

    Args:
        anomaly_detector_arn(str): The Amazon Resource Name (ARN) of the anomaly detector.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.lookoutmetric.test_anomaly_detector.back(
                ctx, anomaly_detector_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.lookoutmetric.test_anomaly_detector.back anomaly_detector_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="lookoutmetrics",
        operation="back_test_anomaly_detector",
        op_kwargs={"AnomalyDetectorArn": anomaly_detector_arn},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def resend(hub, ctx, domain_name: str = None) -> Dict:
    r"""
    **Autogenerated function**

    For operations that require confirmation that the email address for the registrant contact is valid, such as registering a new domain, this operation resends the confirmation email to the current email address for the registrant contact.

    Args:
        domain_name(str, optional): The name of the domain for which you want Route 53 to resend a confirmation email to the registrant contact. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.route53domain.contact_reachability.email.resend(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.route53domain.contact_reachability.email.resend
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="route53domains",
        operation="resend_contact_reachability_email",
        op_kwargs={"domainName": domain_name},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get(
    hub, ctx, name: str, version: str, resource_type: str, export_type: str
) -> Dict:
    r"""
    **Autogenerated function**

    Exports the contents of a Amazon Lex resource in a specified format.

    Args:
        name(str): The name of the bot to export.
        version(str): The version of the bot to export.
        resource_type(str): The type of resource to export. .
        export_type(str): The format of the exported data.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.lex_model.export.get(
                ctx, name=value, version=value, resource_type=value, export_type=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.lex_model.export.get name=value, version=value, resource_type=value, export_type=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="lex-models",
        operation="get_export",
        op_kwargs={
            "name": name,
            "version": version,
            "resourceType": resource_type,
            "exportType": export_type,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__virtualname__ = "slot_type"


async def list_all(
    hub,
    ctx,
    locale_id: str,
    sort_by: Dict = None,
    max_results: int = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Gets a list of built-in slot types that meet the specified criteria.

    Args:
        locale_id(str): The identifier of the language and locale of the slot types to list. The string must match one of the supported locales. For more information, see Supported languages.
        sort_by(Dict, optional): Determines the sort order for the response from the ListBuiltInSlotTypes operation. You can choose to sort by the slot type signature in either ascending or descending order. Defaults to None.
        max_results(int, optional): The maximum number of built-in slot types to return in each page of results. If there are fewer results than the max page size, only the actual number of results are returned. Defaults to None.
        next_token(str, optional): If the response from the ListBuiltInSlotTypes operation contains more results than specified in the maxResults parameter, a token is returned in the response. Use that token in the nextToken parameter to return the next page of results. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.lexv2_model.built_in_.slot_type_.list_all(ctx, locale_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.lexv2_model.built_in_.slot_type.list_all locale_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="lexv2-models",
        operation="list_built_in_slot_types",
        op_kwargs={
            "localeId": locale_id,
            "sortBy": sort_by,
            "maxResults": max_results,
            "nextToken": next_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__func_alias__ = {"list_": "list"}


async def list_(hub, ctx, resource_arn: str, next_token: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Returns a list of tags that are applied to the specified AWS X-Ray group or sampling rule.

    Args:
        resource_arn(str): The Amazon Resource Number (ARN) of an X-Ray group or sampling rule.
        next_token(str, optional): A pagination token. If multiple pages of results are returned, use the NextToken value returned with the current page of results as the value of this parameter to get the next page of results. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.xray.tags_for_resource.list(ctx, resource_arn=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.xray.tags_for_resource.list resource_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="xray",
        operation="list_tags_for_resource",
        op_kwargs={"ResourceARN": resource_arn, "NextToken": next_token},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(
    hub,
    ctx,
    function_name: str,
    name: str,
    function_version: str,
    description: str = None,
    routing_config: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates an alias for a Lambda function version. Use aliases to provide clients with a function identifier that you can update to invoke a different version. You can also map an alias to split invocation requests between two versions. Use the RoutingConfig parameter to specify a second version and the percentage of invocation requests that it receives.

    Args:
        function_name(str): The name of the Lambda function.  Name formats     Function name - MyFunction.    Function ARN - arn:aws:lambda:us-west-2:123456789012:function:MyFunction.    Partial ARN - 123456789012:function:MyFunction.   The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.
        name(str): The name of the alias.
        function_version(str): The function version that the alias invokes.
        description(str, optional): A description of the alias. Defaults to None.
        routing_config(Dict, optional): The routing configuration of the alias. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.lambda_.alias.create(
                ctx, function_name=value, name=value, function_version=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.lambda_.alias.create function_name=value, name=value, function_version=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="lambda",
        operation="create_alias",
        op_kwargs={
            "FunctionName": function_name,
            "Name": name,
            "FunctionVersion": function_version,
            "Description": description,
            "RoutingConfig": routing_config,
        },
    )


async def delete(hub, ctx, function_name: str, name: str) -> None:
    r"""
    **Autogenerated function**

    Deletes a Lambda function alias.

    Args:
        function_name(str): The name of the Lambda function.  Name formats     Function name - MyFunction.    Function ARN - arn:aws:lambda:us-west-2:123456789012:function:MyFunction.    Partial ARN - 123456789012:function:MyFunction.   The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.
        name(str): The name of the alias.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.lambda_.alias.delete(ctx, function_name=value, name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.lambda_.alias.delete function_name=value, name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="lambda",
        operation="delete_alias",
        op_kwargs={"FunctionName": function_name, "Name": name},
    )


async def get(hub, ctx, function_name: str, name: str) -> Dict:
    r"""
    **Autogenerated function**

    Returns details about a Lambda function alias.

    Args:
        function_name(str): The name of the Lambda function.  Name formats     Function name - MyFunction.    Function ARN - arn:aws:lambda:us-west-2:123456789012:function:MyFunction.    Partial ARN - 123456789012:function:MyFunction.   The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.
        name(str): The name of the alias.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.lambda_.alias.get(ctx, function_name=value, name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.lambda_.alias.get function_name=value, name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="lambda",
        operation="get_alias",
        op_kwargs={"FunctionName": function_name, "Name": name},
    )


async def list_all(
    hub,
    ctx,
    function_name: str,
    function_version: str = None,
    marker: str = None,
    max_items: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns a list of aliases for a Lambda function.

    Args:
        function_name(str): The name of the Lambda function.  Name formats     Function name - MyFunction.    Function ARN - arn:aws:lambda:us-west-2:123456789012:function:MyFunction.    Partial ARN - 123456789012:function:MyFunction.   The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.
        function_version(str, optional): Specify a function version to only list aliases that invoke that version. Defaults to None.
        marker(str, optional): Specify the pagination token that's returned by a previous request to retrieve the next page of results. Defaults to None.
        max_items(int, optional): Limit the number of aliases returned. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.lambda_.alias.list_all(ctx, function_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.lambda_.alias.list_all function_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="lambda",
        operation="list_aliases",
        op_kwargs={
            "FunctionName": function_name,
            "FunctionVersion": function_version,
            "Marker": marker,
            "MaxItems": max_items,
        },
    )


async def update(
    hub,
    ctx,
    function_name: str,
    name: str,
    function_version: str = None,
    description: str = None,
    routing_config: Dict = None,
    revision_id: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Updates the configuration of a Lambda function alias.

    Args:
        function_name(str): The name of the Lambda function.  Name formats     Function name - MyFunction.    Function ARN - arn:aws:lambda:us-west-2:123456789012:function:MyFunction.    Partial ARN - 123456789012:function:MyFunction.   The length constraint applies only to the full ARN. If you specify only the function name, it is limited to 64 characters in length.
        name(str): The name of the alias.
        function_version(str, optional): The function version that the alias invokes. Defaults to None.
        description(str, optional): A description of the alias. Defaults to None.
        routing_config(Dict, optional): The routing configuration of the alias. Defaults to None.
        revision_id(str, optional): Only update the alias if the revision ID matches the ID that's specified. Use this option to avoid modifying an alias that has changed since you last read it. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.lambda_.alias.update(ctx, function_name=value, name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.lambda_.alias.update function_name=value, name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="lambda",
        operation="update_alias",
        op_kwargs={
            "FunctionName": function_name,
            "Name": name,
            "FunctionVersion": function_version,
            "Description": description,
            "RoutingConfig": routing_config,
            "RevisionId": revision_id,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    instance_id: str,
    name: str,
    hours_of_operation_id: str,
    description: str = None,
    outbound_caller_config: Dict = None,
    max_contacts: int = None,
    quick_connect_ids: List = None,
    tags: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    This API is in preview release for Amazon Connect and is subject to change. Creates a new queue for the specified Amazon Connect instance.

    Args:
        instance_id(str): The identifier of the Amazon Connect instance. You can find the instanceId in the ARN of the instance.
        name(str): The name of the queue.
        description(str, optional): The description of the queue. Defaults to None.
        outbound_caller_config(Dict, optional): The outbound caller ID name, number, and outbound whisper flow. Defaults to None.
        hours_of_operation_id(str): The identifier for the hours of operation.
        max_contacts(int, optional): The maximum number of contacts that can be in the queue before it is considered full. Defaults to None.
        quick_connect_ids(List, optional): The quick connects available to agents who are working the queue. Defaults to None.
        tags(Dict, optional): One or more tags. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.connect.queue.init.create(
                ctx, instance_id=value, name=value, hours_of_operation_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.connect.queue.init.create instance_id=value, name=value, hours_of_operation_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="connect",
        operation="create_queue",
        op_kwargs={
            "InstanceId": instance_id,
            "Name": name,
            "Description": description,
            "OutboundCallerConfig": outbound_caller_config,
            "HoursOfOperationId": hours_of_operation_id,
            "MaxContacts": max_contacts,
            "QuickConnectIds": quick_connect_ids,
            "Tags": tags,
        },
    )


async def describe(hub, ctx, instance_id: str, queue_id: str) -> Dict:
    r"""
    **Autogenerated function**

    This API is in preview release for Amazon Connect and is subject to change. Describes the specified queue.

    Args:
        instance_id(str): The identifier of the Amazon Connect instance. You can find the instanceId in the ARN of the instance.
        queue_id(str): The identifier for the queue.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.connect.queue.init.describe(
                ctx, instance_id=value, queue_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.connect.queue.init.describe instance_id=value, queue_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="connect",
        operation="describe_queue",
        op_kwargs={"InstanceId": instance_id, "QueueId": queue_id},
    )


async def list_all(
    hub,
    ctx,
    instance_id: str,
    queue_types: List = None,
    next_token: str = None,
    max_results: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Provides information about the queues for the specified Amazon Connect instance. If you do not specify a QueueTypes parameter, both standard and agent queues are returned. This might cause an unexpected truncation of results if you have more than 1000 agents and you limit the number of results of the API call in code. For more information about queues, see Queues: Standard and Agent in the Amazon Connect Administrator Guide.

    Args:
        instance_id(str): The identifier of the Amazon Connect instance. You can find the instanceId in the ARN of the instance.
        queue_types(List, optional): The type of queue. Defaults to None.
        next_token(str, optional): The token for the next set of results. Use the value returned in the previous response in the next request to retrieve the next set of results. Defaults to None.
        max_results(int, optional): The maximum number of results to return per page. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.connect.queue.init.list_all(ctx, instance_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.connect.queue.init.list_all instance_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="connect",
        operation="list_queues",
        op_kwargs={
            "InstanceId": instance_id,
            "QueueTypes": queue_types,
            "NextToken": next_token,
            "MaxResults": max_results,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get_all(
    hub, ctx, query_execution_id: str, next_token: str = None, max_results: int = None
) -> Dict:
    r"""
    **Autogenerated function**

    Streams the results of a single query execution specified by QueryExecutionId from the Athena query results location in Amazon S3. For more information, see Query Results in the Amazon Athena User Guide. This request does not execute the query but returns results. Use StartQueryExecution to run a query. To stream query results successfully, the IAM principal with permission to call GetQueryResults also must have permissions to the Amazon S3 GetObject action for the Athena query results location.  IAM principals with permission to the Amazon S3 GetObject action for the query results location are able to retrieve query results from Amazon S3 even if permission to the GetQueryResults action is denied. To restrict user or role access, ensure that Amazon S3 permissions to the Athena query location are denied.

    Args:
        query_execution_id(str): The unique ID of the query execution.
        next_token(str, optional): A token generated by the Athena service that specifies where to continue pagination if a previous request was truncated. To obtain the next set of pages, pass in the NextToken from the response object of the previous page call. Defaults to None.
        max_results(int, optional): The maximum number of results (rows) to return in this request. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.athena.query.result.get_all(ctx, query_execution_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.athena.query.result.get_all query_execution_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="athena",
        operation="get_query_results",
        op_kwargs={
            "QueryExecutionId": query_execution_id,
            "NextToken": next_token,
            "MaxResults": max_results,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def get(
    hub,
    ctx,
    name: str,
    policy: str = None,
    policy_arns: List = None,
    duration_seconds: int = None,
    tags: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns a set of temporary security credentials (consisting of an access key ID, a secret access key, and a security token) for a federated user. A typical use is in a proxy application that gets temporary security credentials on behalf of distributed applications inside a corporate network. You must call the GetFederationToken operation using the long-term security credentials of an IAM user. As a result, this call is appropriate in contexts where those credentials can be safely stored, usually in a server-based application. For a comparison of GetFederationToken with the other API operations that produce temporary credentials, see Requesting Temporary Security Credentials and Comparing the STS API operations in the IAM User Guide.  You can create a mobile-based or browser-based app that can authenticate users using a web identity provider like Login with Amazon, Facebook, Google, or an OpenID Connect-compatible identity provider. In this case, we recommend that you use Amazon Cognito or AssumeRoleWithWebIdentity. For more information, see Federation Through a Web-based Identity Provider in the IAM User Guide.  You can also call GetFederationToken using the security credentials of an Amazon Web Services account root user, but we do not recommend it. Instead, we recommend that you create an IAM user for the purpose of the proxy application. Then attach a policy to the IAM user that limits federated users to only the actions and resources that they need to access. For more information, see IAM Best Practices in the IAM User Guide.   Session duration  The temporary credentials are valid for the specified duration, from 900 seconds (15 minutes) up to a maximum of 129,600 seconds (36 hours). The default session duration is 43,200 seconds (12 hours). Temporary credentials that are obtained by using Amazon Web Services account root user credentials have a maximum duration of 3,600 seconds (1 hour).  Permissions  You can use the temporary credentials created by GetFederationToken in any Amazon Web Services service except the following:   You cannot call any IAM operations using the CLI or the Amazon Web Services API.    You cannot call any STS operations except GetCallerIdentity.   You must pass an inline or managed session policy to this operation. You can pass a single JSON policy document to use as an inline session policy. You can also specify up to 10 managed policies to use as managed session policies. The plaintext that you use for both inline and managed session policies can't exceed 2,048 characters. Though the session policy parameters are optional, if you do not pass a policy, then the resulting federated user session has no permissions. When you pass session policies, the session permissions are the intersection of the IAM user policies and the session policies that you pass. This gives you a way to further restrict the permissions for a federated user. You cannot use session policies to grant more permissions than those that are defined in the permissions policy of the IAM user. For more information, see Session Policies in the IAM User Guide. For information about using GetFederationToken to create temporary security credentials, see GetFederationToken—Federation Through a Custom Identity Broker.  You can use the credentials to access a resource that has a resource-based policy. If that policy specifically references the federated user session in the Principal element of the policy, the session has the permissions allowed by the policy. These permissions are granted in addition to the permissions granted by the session policies.  Tags  (Optional) You can pass tag key-value pairs to your session. These are called session tags. For more information about session tags, see Passing Session Tags in STS in the IAM User Guide.  You can create a mobile-based or browser-based app that can authenticate users using a web identity provider like Login with Amazon, Facebook, Google, or an OpenID Connect-compatible identity provider. In this case, we recommend that you use Amazon Cognito or AssumeRoleWithWebIdentity. For more information, see Federation Through a Web-based Identity Provider in the IAM User Guide.  You can also call GetFederationToken using the security credentials of an Amazon Web Services account root user, but we do not recommend it. Instead, we recommend that you create an IAM user for the purpose of the proxy application. Then attach a policy to the IAM user that limits federated users to only the actions and resources that they need to access. For more information, see IAM Best Practices in the IAM User Guide.   Session duration  The temporary credentials are valid for the specified duration, from 900 seconds (15 minutes) up to a maximum of 129,600 seconds (36 hours). The default session duration is 43,200 seconds (12 hours). Temporary credentials that are obtained by using Amazon Web Services account root user credentials have a maximum duration of 3,600 seconds (1 hour).  Permissions  You can use the temporary credentials created by GetFederationToken in any Amazon Web Services service except the following:   You cannot call any IAM operations using the CLI or the Amazon Web Services API.    You cannot call any STS operations except GetCallerIdentity.   You must pass an inline or managed session policy to this operation. You can pass a single JSON policy document to use as an inline session policy. You can also specify up to 10 managed policies to use as managed session policies. The plain text that you use for both inline and managed session policies can't exceed 2,048 characters. Though the session policy parameters are optional, if you do not pass a policy, then the resulting federated user session has no permissions. When you pass session policies, the session permissions are the intersection of the IAM user policies and the session policies that you pass. This gives you a way to further restrict the permissions for a federated user. You cannot use session policies to grant more permissions than those that are defined in the permissions policy of the IAM user. For more information, see Session Policies in the IAM User Guide. For information about using GetFederationToken to create temporary security credentials, see GetFederationToken—Federation Through a Custom Identity Broker.  You can use the credentials to access a resource that has a resource-based policy. If that policy specifically references the federated user session in the Principal element of the policy, the session has the permissions allowed by the policy. These permissions are granted in addition to the permissions granted by the session policies.  Tags  (Optional) You can pass tag key-value pairs to your session. These are called session tags. For more information about session tags, see Passing Session Tags in STS in the IAM User Guide. An administrator must grant you the permissions necessary to pass session tags. The administrator can also create granular permissions to allow you to pass only specific session tags. For more information, see Tutorial: Using Tags for Attribute-Based Access Control in the IAM User Guide. Tag key–value pairs are not case sensitive, but case is preserved. This means that you cannot have separate Department and department tag keys. Assume that the user that you are federating has the Department=Marketing tag and you pass the department=engineering session tag. Department and department are not saved as separate tags, and the session tag passed in the request takes precedence over the user tag.

    Args:
        name(str): The name of the federated user. The name is used as an identifier for the temporary security credentials (such as Bob). For example, you can reference the federated user name in a resource-based policy, such as in an Amazon S3 bucket policy. The regex used to validate this parameter is a string of characters consisting of upper- and lower-case alphanumeric characters with no spaces. You can also include underscores or any of the following characters: =,.@-.
        policy(str, optional): An IAM policy in JSON format that you want to use as an inline session policy. You must pass an inline or managed session policy to this operation. You can pass a single JSON policy document to use as an inline session policy. You can also specify up to 10 managed policies to use as managed session policies. This parameter is optional. However, if you do not pass any session policies, then the resulting federated user session has no permissions. When you pass session policies, the session permissions are the intersection of the IAM user policies and the session policies that you pass. This gives you a way to further restrict the permissions for a federated user. You cannot use session policies to grant more permissions than those that are defined in the permissions policy of the IAM user. For more information, see Session Policies in the IAM User Guide. The resulting credentials can be used to access a resource that has a resource-based policy. If that policy specifically references the federated user session in the Principal element of the policy, the session has the permissions allowed by the policy. These permissions are granted in addition to the permissions that are granted by the session policies. The plaintext that you use for both inline and managed session policies can't exceed 2,048 characters. The JSON policy characters can be any ASCII character from the space character to the end of the valid character list (\u0020 through \u00FF). It can also include the tab (\u0009), linefeed (\u000A), and carriage return (\u000D) characters.  An Amazon Web Services conversion compresses the passed session policies and session tags into a packed binary format that has a separate limit. Your request can fail for this limit even if your plaintext meets the other requirements. The PackedPolicySize response element indicates by percentage how close the policies and tags for your request are to the upper size limit.  . Defaults to None.
        policy_arns(List, optional): The Amazon Resource Names (ARNs) of the IAM managed policies that you want to use as a managed session policy. The policies must exist in the same account as the IAM user that is requesting federated access. You must pass an inline or managed session policy to this operation. You can pass a single JSON policy document to use as an inline session policy. You can also specify up to 10 managed policies to use as managed session policies. The plaintext that you use for both inline and managed session policies can't exceed 2,048 characters. You can provide up to 10 managed policy ARNs. For more information about ARNs, see Amazon Resource Names (ARNs) and Amazon Web Services Service Namespaces in the Amazon Web Services General Reference. This parameter is optional. However, if you do not pass any session policies, then the resulting federated user session has no permissions. When you pass session policies, the session permissions are the intersection of the IAM user policies and the session policies that you pass. This gives you a way to further restrict the permissions for a federated user. You cannot use session policies to grant more permissions than those that are defined in the permissions policy of the IAM user. For more information, see Session Policies in the IAM User Guide. The resulting credentials can be used to access a resource that has a resource-based policy. If that policy specifically references the federated user session in the Principal element of the policy, the session has the permissions allowed by the policy. These permissions are granted in addition to the permissions that are granted by the session policies.  An Amazon Web Services conversion compresses the passed session policies and session tags into a packed binary format that has a separate limit. Your request can fail for this limit even if your plaintext meets the other requirements. The PackedPolicySize response element indicates by percentage how close the policies and tags for your request are to the upper size limit.  . Defaults to None.
        duration_seconds(int, optional): The duration, in seconds, that the session should last. Acceptable durations for federation sessions range from 900 seconds (15 minutes) to 129,600 seconds (36 hours), with 43,200 seconds (12 hours) as the default. Sessions obtained using Amazon Web Services account root user credentials are restricted to a maximum of 3,600 seconds (one hour). If the specified duration is longer than one hour, the session obtained by using root user credentials defaults to one hour. Defaults to None.
        tags(List, optional): A list of session tags. Each session tag consists of a key name and an associated value. For more information about session tags, see Passing Session Tags in STS in the IAM User Guide. This parameter is optional. You can pass up to 50 session tags. The plaintext session tag keys can’t exceed 128 characters and the values can’t exceed 256 characters. For these and additional limits, see IAM and STS Character Limits in the IAM User Guide.  An Amazon Web Services conversion compresses the passed session policies and session tags into a packed binary format that has a separate limit. Your request can fail for this limit even if your plaintext meets the other requirements. The PackedPolicySize response element indicates by percentage how close the policies and tags for your request are to the upper size limit.   You can pass a session tag with the same key as a tag that is already attached to the user you are federating. When you do, session tags override a user tag with the same key.  Tag key–value pairs are not case sensitive, but case is preserved. This means that you cannot have separate Department and department tag keys. Assume that the role has the Department=Marketing tag and you pass the department=engineering session tag. Department and department are not saved as separate tags, and the session tag passed in the request takes precedence over the role tag. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sts.federation_token.get(ctx, name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sts.federation_token.get name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sts",
        operation="get_federation_token",
        op_kwargs={
            "Name": name,
            "Policy": policy,
            "PolicyArns": policy_arns,
            "DurationSeconds": duration_seconds,
            "Tags": tags,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def associate(hub, ctx, id_: str, iot_certificate_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Associates a wireless gateway with a certificate.

    Args:
        id_(str): The ID of the resource to update.
        iot_certificate_id(str): The ID of the certificate to associate with the wireless gateway.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iotwireless.wireless.gateway.with_.certificate.associate(
                ctx, id_=value, iot_certificate_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iotwireless.wireless.gateway.with_.certificate.associate id_=value, iot_certificate_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iotwireless",
        operation="associate_wireless_gateway_with_certificate",
        op_kwargs={"Id": id_, "IotCertificateId": iot_certificate_id},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def register(hub, ctx, load_balancer_name: str, instances: List) -> Dict:
    r"""
    **Autogenerated function**

    Adds the specified instances to the specified load balancer. The instance must be a running instance in the same network as the load balancer (EC2-Classic or the same VPC). If you have EC2-Classic instances and a load balancer in a VPC with ClassicLink enabled, you can link the EC2-Classic instances to that VPC and then register the linked EC2-Classic instances with the load balancer in the VPC. Note that RegisterInstanceWithLoadBalancer completes when the request has been registered. Instance registration takes a little time to complete. To check the state of the registered instances, use DescribeLoadBalancers or DescribeInstanceHealth. After the instance is registered, it starts receiving traffic and requests from the load balancer. Any instance that is not in one of the Availability Zones registered for the load balancer is moved to the OutOfService state. If an Availability Zone is added to the load balancer later, any instances registered with the load balancer move to the InService state. To deregister instances from a load balancer, use DeregisterInstancesFromLoadBalancer. For more information, see Register or De-Register EC2 Instances in the Classic Load Balancers Guide.

    Args:
        load_balancer_name(str): The name of the load balancer.
        instances(List): The IDs of the instances.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elb.instance.with_load_balancer.register(
                ctx, load_balancer_name=value, instances=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elb.instance.with_load_balancer.register load_balancer_name=value, instances=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elb",
        operation="register_instances_with_load_balancer",
        op_kwargs={"LoadBalancerName": load_balancer_name, "Instances": instances},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def list_all(
    hub,
    ctx,
    directory_arn: str,
    object_reference: Dict,
    filter_attribute_ranges: List = None,
    filter_typed_link: Dict = None,
    next_token: str = None,
    max_results: int = None,
    consistency_level: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns a paginated list of all the incoming TypedLinkSpecifier information for an object. It also supports filtering by typed link facet and identity attributes. For more information, see Typed Links.

    Args:
        directory_arn(str): The Amazon Resource Name (ARN) of the directory where you want to list the typed links.
        object_reference(Dict): Reference that identifies the object whose attributes will be listed.
        filter_attribute_ranges(List, optional): Provides range filters for multiple attributes. When providing ranges to typed link selection, any inexact ranges must be specified at the end. Any attributes that do not have a range specified are presumed to match the entire range. Defaults to None.
        filter_typed_link(Dict, optional): Filters are interpreted in the order of the attributes on the typed link facet, not the order in which they are supplied to any API calls. Defaults to None.
        next_token(str, optional): The pagination token. Defaults to None.
        max_results(int, optional): The maximum number of results to retrieve. Defaults to None.
        consistency_level(str, optional): The consistency level to execute the request at. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.clouddirectory.incoming_typed_link.list_all(
                ctx, directory_arn=value, object_reference=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.clouddirectory.incoming_typed_link.list_all directory_arn=value, object_reference=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="clouddirectory",
        operation="list_incoming_typed_links",
        op_kwargs={
            "DirectoryArn": directory_arn,
            "ObjectReference": object_reference,
            "FilterAttributeRanges": filter_attribute_ranges,
            "FilterTypedLink": filter_typed_link,
            "NextToken": next_token,
            "MaxResults": max_results,
            "ConsistencyLevel": consistency_level,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def deactivate(hub, ctx, user_name: str, serial_number: str) -> None:
    r"""
    **Autogenerated function**

    Deactivates the specified MFA device and removes it from association with the user name for which it was originally enabled. For more information about creating and working with virtual MFA devices, see Enabling a virtual multi-factor authentication (MFA) device in the IAM User Guide.

    Args:
        user_name(str): The name of the user whose MFA device you want to deactivate. This parameter allows (through its regex pattern) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-.
        serial_number(str): The serial number that uniquely identifies the MFA device. For virtual MFA devices, the serial number is the device ARN. This parameter allows (through its regex pattern) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: =,.@:/-.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iam.mfa_device.init.deactivate(
                ctx, user_name=value, serial_number=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iam.mfa_device.init.deactivate user_name=value, serial_number=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iam",
        operation="deactivate_mfa_device",
        op_kwargs={"UserName": user_name, "SerialNumber": serial_number},
    )


async def enable(
    hub,
    ctx,
    user_name: str,
    serial_number: str,
    authentication_code1: str,
    authentication_code2: str,
) -> None:
    r"""
    **Autogenerated function**

    Enables the specified MFA device and associates it with the specified IAM user. When enabled, the MFA device is required for every subsequent login by the IAM user associated with the device.

    Args:
        user_name(str): The name of the IAM user for whom you want to enable the MFA device. This parameter allows (through its regex pattern) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-.
        serial_number(str): The serial number that uniquely identifies the MFA device. For virtual MFA devices, the serial number is the device ARN. This parameter allows (through its regex pattern) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: =,.@:/-.
        authentication_code1(str): An authentication code emitted by the device.  The format for this parameter is a string of six digits.  Submit your request immediately after generating the authentication codes. If you generate the codes and then wait too long to submit the request, the MFA device successfully associates with the user but the MFA device becomes out of sync. This happens because time-based one-time passwords (TOTP) expire after a short period of time. If this happens, you can resync the device. .
        authentication_code2(str): A subsequent authentication code emitted by the device. The format for this parameter is a string of six digits.  Submit your request immediately after generating the authentication codes. If you generate the codes and then wait too long to submit the request, the MFA device successfully associates with the user but the MFA device becomes out of sync. This happens because time-based one-time passwords (TOTP) expire after a short period of time. If this happens, you can resync the device. .
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iam.mfa_device.init.enable(
                ctx,
                user_name=value,
                serial_number=value,
                authentication_code1=value,
                authentication_code2=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iam.mfa_device.init.enable user_name=value, serial_number=value, authentication_code1=value, authentication_code2=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iam",
        operation="enable_mfa_device",
        op_kwargs={
            "UserName": user_name,
            "SerialNumber": serial_number,
            "AuthenticationCode1": authentication_code1,
            "AuthenticationCode2": authentication_code2,
        },
    )


async def list_all(
    hub, ctx, user_name: str = None, marker: str = None, max_items: int = None
) -> Dict:
    r"""
    **Autogenerated function**

    Lists the MFA devices for an IAM user. If the request includes a IAM user name, then this operation lists all the MFA devices associated with the specified user. If you do not specify a user name, IAM determines the user name implicitly based on the Amazon Web Services access key ID signing the request for this operation. You can paginate the results using the MaxItems and Marker parameters.

    Args:
        user_name(str, optional): The name of the user whose MFA devices you want to list. This parameter allows (through its regex pattern) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-. Defaults to None.
        marker(str, optional): Use this parameter only when paginating results and only after you receive a response indicating that the results are truncated. Set it to the value of the Marker element in the response that you received to indicate where the next call should start. Defaults to None.
        max_items(int, optional): Use this only when paginating results to indicate the maximum number of items you want in the response. If additional items exist beyond the maximum you specify, the IsTruncated response element is true. If you do not include this parameter, the number of items defaults to 100. Note that IAM might return fewer results, even when there are more results available. In that case, the IsTruncated response element returns true, and Marker contains a value to include in the subsequent call that tells the service where to continue from. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iam.mfa_device.init.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iam.mfa_device.init.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iam",
        operation="list_mfa_devices",
        op_kwargs={"UserName": user_name, "Marker": marker, "MaxItems": max_items},
    )


async def resync(
    hub,
    ctx,
    user_name: str,
    serial_number: str,
    authentication_code1: str,
    authentication_code2: str,
) -> None:
    r"""
    **Autogenerated function**

    Synchronizes the specified MFA device with its IAM resource object on the Amazon Web Services servers. For more information about creating and working with virtual MFA devices, see Using a virtual MFA device in the IAM User Guide.

    Args:
        user_name(str): The name of the user whose MFA device you want to resynchronize. This parameter allows (through its regex pattern) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-.
        serial_number(str): Serial number that uniquely identifies the MFA device. This parameter allows (through its regex pattern) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-.
        authentication_code1(str): An authentication code emitted by the device. The format for this parameter is a sequence of six digits.
        authentication_code2(str): A subsequent authentication code emitted by the device. The format for this parameter is a sequence of six digits.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iam.mfa_device.init.resync(
                ctx,
                user_name=value,
                serial_number=value,
                authentication_code1=value,
                authentication_code2=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iam.mfa_device.init.resync user_name=value, serial_number=value, authentication_code1=value, authentication_code2=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iam",
        operation="resync_mfa_device",
        op_kwargs={
            "UserName": user_name,
            "SerialNumber": serial_number,
            "AuthenticationCode1": authentication_code1,
            "AuthenticationCode2": authentication_code2,
        },
    )


async def tag(hub, ctx, serial_number: str, tags: List) -> None:
    r"""
    **Autogenerated function**

    Adds one or more tags to an IAM virtual multi-factor authentication (MFA) device. If a tag with the same key name already exists, then that tag is overwritten with the new value. A tag consists of a key name and an associated value. By assigning tags to your resources, you can do the following:    Administrative grouping and discovery - Attach tags to resources to aid in organization and search. For example, you could search for all resources with the key name Project and the value MyImportantProject. Or search for all resources with the key name Cost Center and the value 41200.     Access control - Include tags in IAM user-based and resource-based policies. You can use tags to restrict access to only an IAM virtual MFA device that has a specified tag attached. For examples of policies that show how to use tags to control access, see Control access using IAM tags in the IAM User Guide.      If any one of the tags is invalid or if you exceed the allowed maximum number of tags, then the entire request fails and the resource is not created. For more information about tagging, see Tagging IAM resources in the IAM User Guide.   Amazon Web Services always interprets the tag Value as a single string. If you need to store an array, you can store comma-separated values in the string. However, you must interpret the value in your code.

    Args:
        serial_number(str): The unique identifier for the IAM virtual MFA device to which you want to add tags. For virtual MFA devices, the serial number is the same as the ARN. This parameter allows (through its regex pattern) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-.
        tags(List): The list of tags that you want to attach to the IAM virtual MFA device. Each tag consists of a key name and an associated value.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iam.mfa_device.init.tag(ctx, serial_number=value, tags=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iam.mfa_device.init.tag serial_number=value, tags=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iam",
        operation="tag_mfa_device",
        op_kwargs={"SerialNumber": serial_number, "Tags": tags},
    )


async def untag(hub, ctx, serial_number: str, tag_keys: List) -> None:
    r"""
    **Autogenerated function**

    Removes the specified tags from the IAM virtual multi-factor authentication (MFA) device. For more information about tagging, see Tagging IAM resources in the IAM User Guide.

    Args:
        serial_number(str): The unique identifier for the IAM virtual MFA device from which you want to remove tags. For virtual MFA devices, the serial number is the same as the ARN. This parameter allows (through its regex pattern) a string of characters consisting of upper and lowercase alphanumeric characters with no spaces. You can also include any of the following characters: _+=,.@-.
        tag_keys(List): A list of key names as a simple array of strings. The tags with matching keys are removed from the specified instance profile.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iam.mfa_device.init.untag(
                ctx, serial_number=value, tag_keys=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iam.mfa_device.init.untag serial_number=value, tag_keys=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iam",
        operation="untag_mfa_device",
        op_kwargs={"SerialNumber": serial_number, "TagKeys": tag_keys},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def deregister(hub, ctx, device_fleet_name: str, device_names: List) -> None:
    r"""
    **Autogenerated function**

    Deregisters the specified devices. After you deregister a device, you will need to re-register the devices.

    Args:
        device_fleet_name(str): The name of the fleet the devices belong to.
        device_names(List): The unique IDs of the devices.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sagemaker.device.init.deregister(
                ctx, device_fleet_name=value, device_names=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sagemaker.device.init.deregister device_fleet_name=value, device_names=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sagemaker",
        operation="deregister_devices",
        op_kwargs={"DeviceFleetName": device_fleet_name, "DeviceNames": device_names},
    )


async def describe(
    hub, ctx, device_name: str, device_fleet_name: str, next_token: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Describes the device.

    Args:
        next_token(str, optional): Next token of device description. Defaults to None.
        device_name(str): The unique ID of the device.
        device_fleet_name(str): The name of the fleet the devices belong to.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sagemaker.device.init.describe(
                ctx, device_name=value, device_fleet_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sagemaker.device.init.describe device_name=value, device_fleet_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sagemaker",
        operation="describe_device",
        op_kwargs={
            "NextToken": next_token,
            "DeviceName": device_name,
            "DeviceFleetName": device_fleet_name,
        },
    )


async def list_all(
    hub,
    ctx,
    next_token: str = None,
    max_results: int = None,
    latest_heartbeat_after: str = None,
    model_name: str = None,
    device_fleet_name: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    A list of devices.

    Args:
        next_token(str, optional): The response from the last list when returning a list large enough to need tokening. Defaults to None.
        max_results(int, optional): Maximum number of results to select. Defaults to None.
        latest_heartbeat_after(str, optional): Select fleets where the job was updated after X. Defaults to None.
        model_name(str, optional): A filter that searches devices that contains this name in any of their models. Defaults to None.
        device_fleet_name(str, optional): Filter for fleets containing this name in their device fleet name. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sagemaker.device.init.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sagemaker.device.init.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sagemaker",
        operation="list_devices",
        op_kwargs={
            "NextToken": next_token,
            "MaxResults": max_results,
            "LatestHeartbeatAfter": latest_heartbeat_after,
            "ModelName": model_name,
            "DeviceFleetName": device_fleet_name,
        },
    )


async def register(
    hub, ctx, device_fleet_name: str, devices: List, tags: List = None
) -> None:
    r"""
    **Autogenerated function**

    Register devices.

    Args:
        device_fleet_name(str): The name of the fleet.
        devices(List): A list of devices to register with SageMaker Edge Manager.
        tags(List, optional): The tags associated with devices. Defaults to None.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sagemaker.device.init.register(
                ctx, device_fleet_name=value, devices=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sagemaker.device.init.register device_fleet_name=value, devices=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sagemaker",
        operation="register_devices",
        op_kwargs={
            "DeviceFleetName": device_fleet_name,
            "Devices": devices,
            "Tags": tags,
        },
    )


async def update_multiple(hub, ctx, device_fleet_name: str, devices: List) -> None:
    r"""
    **Autogenerated function**

    Updates one or more devices in a fleet.

    Args:
        device_fleet_name(str): The name of the fleet the devices belong to.
        devices(List): List of devices to register with Edge Manager agent.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sagemaker.device.init.update_multiple(
                ctx, device_fleet_name=value, devices=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sagemaker.device.init.update_multiple device_fleet_name=value, devices=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sagemaker",
        operation="update_devices",
        op_kwargs={"DeviceFleetName": device_fleet_name, "Devices": devices},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def batch(hub, ctx, standards_subscription_arns: List) -> Dict:
    r"""
    **Autogenerated function**

    Disables the standards specified by the provided StandardsSubscriptionArns. For more information, see Security Standards section of the Security Hub User Guide.

    Args:
        standards_subscription_arns(List): The ARNs of the standards subscriptions to disable.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.securityhub.disable_standard.batch(
                ctx, standards_subscription_arns=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.securityhub.disable_standard.batch standards_subscription_arns=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="securityhub",
        operation="batch_disable_standards",
        op_kwargs={"StandardsSubscriptionArns": standards_subscription_arns},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def list_all(
    hub,
    ctx,
    schema_arn: str,
    name: str,
    next_token: str = None,
    max_results: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves attributes attached to the facet.

    Args:
        schema_arn(str): The ARN of the schema where the facet resides.
        name(str): The name of the facet whose attributes will be retrieved.
        next_token(str, optional): The pagination token. Defaults to None.
        max_results(int, optional): The maximum number of results to retrieve. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.clouddirectory.facet.attribute.list_all(
                ctx, schema_arn=value, name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.clouddirectory.facet.attribute.list_all schema_arn=value, name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="clouddirectory",
        operation="list_facet_attributes",
        op_kwargs={
            "SchemaArn": schema_arn,
            "Name": name,
            "NextToken": next_token,
            "MaxResults": max_results,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def list_all(
    hub,
    ctx,
    domain: str,
    repository: str,
    format_: str,
    package: str,
    package_version: str,
    domain_owner: str = None,
    namespace: str = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

     Returns the direct dependencies for a package version. The dependencies are returned as PackageDependency objects. CodeArtifact extracts the dependencies for a package version from the metadata file for the package format (for example, the package.json file for npm packages and the pom.xml file for Maven). Any package version dependencies that are not listed in the configuration file are not returned.

    Args:
        domain(str):  The name of the domain that contains the repository that contains the requested package version dependencies. .
        domain_owner(str, optional):  The 12-digit account number of the AWS account that owns the domain. It does not include dashes or spaces. . Defaults to None.
        repository(str):  The name of the repository that contains the requested package version. .
        format_(str):  The format of the package with the requested dependencies. The valid package types are:     npm: A Node Package Manager (npm) package.     pypi: A Python Package Index (PyPI) package.     maven: A Maven package that contains compiled code in a distributable format, such as a JAR file.   .
        namespace(str, optional):  The namespace of the package. The package component that specifies its namespace depends on its type. For example:     The namespace of a Maven package is its groupId.     The namespace of an npm package is its scope.     A Python package does not contain a corresponding component, so Python packages do not have a namespace.   . Defaults to None.
        package(str):  The name of the package versions' package. .
        package_version(str):  A string that contains the package version (for example, 3.5.2). .
        next_token(str, optional):  The token for the next set of results. Use the value returned in the previous response in the next request to retrieve the next set of results. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codeartifact.package.version.dependency.list_all(
                ctx,
                domain=value,
                repository=value,
                format_=value,
                package=value,
                package_version=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codeartifact.package.version.dependency.list_all domain=value, repository=value, format_=value, package=value, package_version=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codeartifact",
        operation="list_package_version_dependencies",
        op_kwargs={
            "domain": domain,
            "domainOwner": domain_owner,
            "repository": repository,
            "format": format_,
            "namespace": namespace,
            "package": package,
            "packageVersion": package_version,
            "nextToken": next_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def get(
    hub,
    ctx,
    target_id_filters: List = None,
    region_filters: List = None,
    resource_type_filters: List = None,
    tag_key_filters: List = None,
    group_by: List = None,
    max_results: int = None,
    pagination_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns a table that shows counts of resources that are noncompliant with their tag policies. For more information on tag policies, see Tag Policies in the AWS Organizations User Guide.  You can call this operation only from the organization's management account and from the us-east-1 Region. This operation supports pagination, where the response can be sent in multiple pages. You should check the PaginationToken response parameter to determine if there are additional results available to return. Repeat the query, passing the PaginationToken response parameter value as an input to the next request until you recieve a null value. A null value for PaginationToken indicates that there are no more results waiting to be returned.

    Args:
        target_id_filters(List, optional): Specifies target identifiers (usually, specific account IDs) to limit the output by. If you use this parameter, the count of returned noncompliant resources includes only resources with the specified target IDs. Defaults to None.
        region_filters(List, optional): Specifies a list of AWS Regions to limit the output by. If you use this parameter, the count of returned noncompliant resources includes only resources in the specified Regions. Defaults to None.
        resource_type_filters(List, optional): Specifies that you want the response to include information for only resources of the specified types. The format of each resource type is service[:resourceType]. For example, specifying a resource type of ec2 returns all Amazon EC2 resources (which includes EC2 instances). Specifying a resource type of ec2:instance returns only EC2 instances.  The string for each service name and resource type is the same as that embedded in a resource's Amazon Resource Name (ARN). Consult the AWS General Reference for the following:   For a list of service name strings, see AWS Service Namespaces.   For resource type strings, see Example ARNs.   For more information about ARNs, see Amazon Resource Names (ARNs) and AWS Service Namespaces.   You can specify multiple resource types by using a comma separated array. The array can include up to 100 items. Note that the length constraint requirement applies to each resource type filter. . Defaults to None.
        tag_key_filters(List, optional): Specifies that you want the response to include information for only resources that have tags with the specified tag keys. If you use this parameter, the count of returned noncompliant resources includes only resources that have the specified tag keys. Defaults to None.
        group_by(List, optional): Specifies a list of attributes to group the counts of noncompliant resources by. If supplied, the counts are sorted by those attributes. Defaults to None.
        max_results(int, optional): Specifies the maximum number of results to be returned in each page. A query can return fewer than this maximum, even if there are more results still to return. You should always check the PaginationToken response value to see if there are more results. You can specify a minimum of 1 and a maximum value of 100. Defaults to None.
        pagination_token(str, optional): Specifies a PaginationToken response value from a previous request to indicate that you want the next page of results. Leave this parameter empty in your initial request. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.resourcegroupstaggingapi.compliance_summary.get(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.resourcegroupstaggingapi.compliance_summary.get
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="resourcegroupstaggingapi",
        operation="get_compliance_summary",
        op_kwargs={
            "TargetIdFilters": target_id_filters,
            "RegionFilters": region_filters,
            "ResourceTypeFilters": resource_type_filters,
            "TagKeyFilters": tag_key_filters,
            "GroupBy": group_by,
            "MaxResults": max_results,
            "PaginationToken": pagination_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    cache_subnet_group_name: str,
    cache_subnet_group_description: str,
    subnet_ids: List,
    tags: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a new cache subnet group. Use this parameter only when you are creating a cluster in an Amazon Virtual Private Cloud (Amazon VPC).

    Args:
        cache_subnet_group_name(str): A name for the cache subnet group. This value is stored as a lowercase string. Constraints: Must contain no more than 255 alphanumeric characters or hyphens. Example: mysubnetgroup .
        cache_subnet_group_description(str): A description for the cache subnet group.
        subnet_ids(List): A list of VPC subnet IDs for the cache subnet group.
        tags(List, optional): A list of tags to be added to this resource. A tag is a key-value pair. A tag key must be accompanied by a tag value, although null is accepted. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elasticache.cache.subnet_group.create(
                ctx,
                cache_subnet_group_name=value,
                cache_subnet_group_description=value,
                subnet_ids=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elasticache.cache.subnet_group.create cache_subnet_group_name=value, cache_subnet_group_description=value, subnet_ids=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elasticache",
        operation="create_cache_subnet_group",
        op_kwargs={
            "CacheSubnetGroupName": cache_subnet_group_name,
            "CacheSubnetGroupDescription": cache_subnet_group_description,
            "SubnetIds": subnet_ids,
            "Tags": tags,
        },
    )


async def delete(hub, ctx, cache_subnet_group_name: str) -> None:
    r"""
    **Autogenerated function**

    Deletes a cache subnet group.  You cannot delete a default cache subnet group or one that is associated with any clusters.

    Args:
        cache_subnet_group_name(str): The name of the cache subnet group to delete. Constraints: Must contain no more than 255 alphanumeric characters or hyphens.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elasticache.cache.subnet_group.delete(
                ctx, cache_subnet_group_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elasticache.cache.subnet_group.delete cache_subnet_group_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elasticache",
        operation="delete_cache_subnet_group",
        op_kwargs={"CacheSubnetGroupName": cache_subnet_group_name},
    )


async def describe_all(
    hub,
    ctx,
    cache_subnet_group_name: str = None,
    max_records: int = None,
    marker: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns a list of cache subnet group descriptions. If a subnet group name is specified, the list contains only the description of that group. This is applicable only when you have ElastiCache in VPC setup. All ElastiCache clusters now launch in VPC by default.

    Args:
        cache_subnet_group_name(str, optional): The name of the cache subnet group to return details for. Defaults to None.
        max_records(int, optional): The maximum number of records to include in the response. If more records exist than the specified MaxRecords value, a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: minimum 20; maximum 100. Defaults to None.
        marker(str, optional): An optional marker returned from a prior request. Use this marker for pagination of results from this operation. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by MaxRecords. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elasticache.cache.subnet_group.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elasticache.cache.subnet_group.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elasticache",
        operation="describe_cache_subnet_groups",
        op_kwargs={
            "CacheSubnetGroupName": cache_subnet_group_name,
            "MaxRecords": max_records,
            "Marker": marker,
        },
    )


async def modify(
    hub,
    ctx,
    cache_subnet_group_name: str,
    cache_subnet_group_description: str = None,
    subnet_ids: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Modifies an existing cache subnet group.

    Args:
        cache_subnet_group_name(str): The name for the cache subnet group. This value is stored as a lowercase string. Constraints: Must contain no more than 255 alphanumeric characters or hyphens. Example: mysubnetgroup .
        cache_subnet_group_description(str, optional): A description of the cache subnet group. Defaults to None.
        subnet_ids(List, optional): The EC2 subnet IDs for the cache subnet group. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elasticache.cache.subnet_group.modify(
                ctx, cache_subnet_group_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elasticache.cache.subnet_group.modify cache_subnet_group_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elasticache",
        operation="modify_cache_subnet_group",
        op_kwargs={
            "CacheSubnetGroupName": cache_subnet_group_name,
            "CacheSubnetGroupDescription": cache_subnet_group_description,
            "SubnetIds": subnet_ids,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def list_all(
    hub,
    ctx,
    hit_id: str = None,
    assignment_id: str = None,
    next_token: str = None,
    max_results: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

     The ListBonusPayments operation retrieves the amounts of bonuses you have paid to Workers for a given HIT or assignment.

    Args:
        hit_id(str, optional): The ID of the HIT associated with the bonus payments to retrieve. If not specified, all bonus payments for all assignments for the given HIT are returned. Either the HITId parameter or the AssignmentId parameter must be specified. Defaults to None.
        assignment_id(str, optional): The ID of the assignment associated with the bonus payments to retrieve. If specified, only bonus payments for the given assignment are returned. Either the HITId parameter or the AssignmentId parameter must be specified. Defaults to None.
        next_token(str, optional): Pagination token. Defaults to None.
        max_results(int, optional): . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.mturk.bonus.payment.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.mturk.bonus.payment.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="mturk",
        operation="list_bonus_payments",
        op_kwargs={
            "HITId": hit_id,
            "AssignmentId": assignment_id,
            "NextToken": next_token,
            "MaxResults": max_results,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def describe_all(hub, ctx, next_token: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves your account's CloudFormation limits, such as the maximum number of stacks that you can create in your account. For more information about account limits, see CloudFormation Limits in the CloudFormation User Guide.

    Args:
        next_token(str, optional): A string that identifies the next page of limits that you want to retrieve. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudformation.account_limit.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudformation.account_limit.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudformation",
        operation="describe_account_limits",
        op_kwargs={"NextToken": next_token},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def batch(hub, ctx, names: List) -> Dict:
    r"""
    **Autogenerated function**

    Gets information about one or more build projects.

    Args:
        names(List): The names or ARNs of the build projects. To get information about a project shared with your AWS account, its ARN must be specified. You cannot specify a shared project using its name.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codebuild.get.project.batch(ctx, names=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codebuild.get.project.batch names=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codebuild",
        operation="batch_get_projects",
        op_kwargs={"names": names},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def poll(
    hub,
    ctx,
    domain: str,
    task_list: Dict,
    identity: str = None,
    next_page_token: str = None,
    maximum_page_size: int = None,
    reverse_order: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Used by deciders to get a DecisionTask from the specified decision taskList. A decision task may be returned for any open workflow execution that is using the specified task list. The task includes a paginated view of the history of the workflow execution. The decider should use the workflow type and the history to determine how to properly handle the task. This action initiates a long poll, where the service holds the HTTP connection open and responds as soon a task becomes available. If no decision task is available in the specified task list before the timeout of 60 seconds expires, an empty result is returned. An empty result, in this context, means that a DecisionTask is returned, but that the value of taskToken is an empty string.  Deciders should set their client side socket timeout to at least 70 seconds (10 seconds higher than the timeout).   Because the number of workflow history events for a single workflow execution might be very large, the result returned might be split up across a number of pages. To retrieve subsequent pages, make additional calls to PollForDecisionTask using the nextPageToken returned by the initial call. Note that you do not call GetWorkflowExecutionHistory with this nextPageToken. Instead, call PollForDecisionTask again.   Access Control  You can use IAM policies to control this action's access to Amazon SWF resources as follows:   Use a Resource element with the domain name to limit the action to only specified domains.   Use an Action element to allow or deny permission to call this action.   Constrain the taskList.name parameter by using a Condition element with the swf:taskList.name key to allow the action to access only certain task lists.   If the caller doesn't have sufficient permissions to invoke the action, or the parameter values fall outside the specified constraints, the action fails. The associated event attribute's cause parameter is set to OPERATION_NOT_PERMITTED. For details and example IAM policies, see Using IAM to Manage Access to Amazon SWF Workflows in the Amazon SWF Developer Guide.

    Args:
        domain(str): The name of the domain containing the task lists to poll.
        task_list(Dict): Specifies the task list to poll for decision tasks. The specified string must not start or end with whitespace. It must not contain a : (colon), / (slash), | (vertical bar), or any control characters (\u0000-\u001f | \u007f-\u009f). Also, it must not be the literal string arn.
        identity(str, optional): Identity of the decider making the request, which is recorded in the DecisionTaskStarted event in the workflow history. This enables diagnostic tracing when problems arise. The form of this identity is user defined. Defaults to None.
        next_page_token(str, optional): If NextPageToken is returned there are more results available. The value of NextPageToken is a unique pagination token for each page. Make the call again using the returned token to retrieve the next page. Keep all other arguments unchanged. Each pagination token expires after 60 seconds. Using an expired pagination token will return a 400 error: "Specified token has exceeded its maximum lifetime".  The configured maximumPageSize determines how many results can be returned in a single call.   The nextPageToken returned by this action cannot be used with GetWorkflowExecutionHistory to get the next page. You must call PollForDecisionTask again (with the nextPageToken) to retrieve the next page of history records. Calling PollForDecisionTask with a nextPageToken doesn't return a new decision task. . Defaults to None.
        maximum_page_size(int, optional): The maximum number of results that are returned per call. Use nextPageToken to obtain further pages of results.  This is an upper limit only; the actual number of results returned per call may be fewer than the specified maximum. Defaults to None.
        reverse_order(bool, optional): When set to true, returns the events in reverse order. By default the results are returned in ascending order of the eventTimestamp of the events. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.swf.for_.decision_task.poll(ctx, domain=value, task_list=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.swf.for_.decision_task.poll domain=value, task_list=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="swf",
        operation="poll_for_decision_task",
        op_kwargs={
            "domain": domain,
            "taskList": task_list,
            "identity": identity,
            "nextPageToken": next_page_token,
            "maximumPageSize": maximum_page_size,
            "reverseOrder": reverse_order,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def get(hub, ctx, arn: str) -> Dict:
    r"""
    **Autogenerated function**

    Gets information about a unique device type.

    Args:
        arn(str): The device type's ARN.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.devicefarm.device.init.get(ctx, arn=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.devicefarm.device.init.get arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx, service_name="devicefarm", operation="get_device", op_kwargs={"arn": arn}
    )


async def list_all(
    hub, ctx, arn: str = None, next_token: str = None, filters: List = None
) -> Dict:
    r"""
    **Autogenerated function**

    Gets information about unique device types.

    Args:
        arn(str, optional): The Amazon Resource Name (ARN) of the project. Defaults to None.
        next_token(str, optional): An identifier that was returned from the previous call to this operation, which can be used to return the next set of items in the list. Defaults to None.
        filters(List, optional): Used to select a set of devices. A filter is made up of an attribute, an operator, and one or more values.   Attribute: The aspect of a device such as platform or model used as the selection criteria in a device filter. Allowed values include:   ARN: The Amazon Resource Name (ARN) of the device (for example, arn:aws:devicefarm:us-west-2::device:12345Example).   PLATFORM: The device platform. Valid values are ANDROID or IOS.   OS_VERSION: The operating system version (for example, 10.3.2).   MODEL: The device model (for example, iPad 5th Gen).   AVAILABILITY: The current availability of the device. Valid values are AVAILABLE, HIGHLY_AVAILABLE, BUSY, or TEMPORARY_NOT_AVAILABLE.   FORM_FACTOR: The device form factor. Valid values are PHONE or TABLET.   MANUFACTURER: The device manufacturer (for example, Apple).   REMOTE_ACCESS_ENABLED: Whether the device is enabled for remote access. Valid values are TRUE or FALSE.   REMOTE_DEBUG_ENABLED: Whether the device is enabled for remote debugging. Valid values are TRUE or FALSE. Because remote debugging is no longer supported, this attribute is ignored.   INSTANCE_ARN: The Amazon Resource Name (ARN) of the device instance.   INSTANCE_LABELS: The label of the device instance.   FLEET_TYPE: The fleet type. Valid values are PUBLIC or PRIVATE.     Operator: The filter operator.   The EQUALS operator is available for every attribute except INSTANCE_LABELS.   The CONTAINS operator is available for the INSTANCE_LABELS and MODEL attributes.   The IN and NOT_IN operators are available for the ARN, OS_VERSION, MODEL, MANUFACTURER, and INSTANCE_ARN attributes.   The LESS_THAN, GREATER_THAN, LESS_THAN_OR_EQUALS, and GREATER_THAN_OR_EQUALS operators are also available for the OS_VERSION attribute.     Values: An array of one or more filter values.   The IN and NOT_IN operators take a values array that has one or more elements.   The other operators require an array with a single element.   In a request, the AVAILABILITY attribute takes the following values: AVAILABLE, HIGHLY_AVAILABLE, BUSY, or TEMPORARY_NOT_AVAILABLE.    . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.devicefarm.device.init.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.devicefarm.device.init.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="devicefarm",
        operation="list_devices",
        op_kwargs={"arn": arn, "nextToken": next_token, "filters": filters},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


__virtualname__ = "to_object"


async def add(
    hub,
    ctx,
    directory_arn: str,
    schema_facet: Dict,
    object_reference: Dict,
    object_attribute_list: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Adds a new Facet to an object. An object can have more than one facet applied on it.

    Args:
        directory_arn(str): The Amazon Resource Name (ARN) that is associated with the Directory where the object resides. For more information, see arns.
        schema_facet(Dict): Identifiers for the facet that you are adding to the object. See SchemaFacet for details.
        object_attribute_list(List, optional): Attributes on the facet that you are adding to the object. Defaults to None.
        object_reference(Dict): A reference to the object you are adding the specified facet to.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.clouddirectory.facet.to_object_.add(
                ctx, directory_arn=value, schema_facet=value, object_reference=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.clouddirectory.facet.to_object.add directory_arn=value, schema_facet=value, object_reference=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="clouddirectory",
        operation="add_facet_to_object",
        op_kwargs={
            "DirectoryArn": directory_arn,
            "SchemaFacet": schema_facet,
            "ObjectAttributeList": object_attribute_list,
            "ObjectReference": object_reference,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    cluster_name: str,
    nodegroup_name: str,
    subnets: List,
    node_role: str,
    scaling_config: Dict = None,
    disk_size: int = None,
    instance_types: List = None,
    ami_type: str = None,
    remote_access: Dict = None,
    labels: Dict = None,
    taints: List = None,
    tags: Dict = None,
    client_request_token: str = None,
    launch_template: Dict = None,
    update_config: Dict = None,
    capacity_type: str = None,
    version: str = None,
    release_version: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a managed node group for an Amazon EKS cluster. You can only create a node group for your cluster that is equal to the current Kubernetes version for the cluster. All node groups are created with the latest AMI release version for the respective minor Kubernetes version of the cluster, unless you deploy a custom AMI using a launch template. For more information about using launch templates, see Launch template support. An Amazon EKS managed node group is an Amazon EC2 Auto Scaling group and associated Amazon EC2 instances that are managed by Amazon Web Services for an Amazon EKS cluster. Each node group uses a version of the Amazon EKS optimized Amazon Linux 2 AMI. For more information, see Managed Node Groups in the Amazon EKS User Guide.

    Args:
        cluster_name(str): The name of the cluster to create the node group in.
        nodegroup_name(str): The unique name to give your node group.
        scaling_config(Dict, optional): The scaling configuration details for the Auto Scaling group that is created for your node group. Defaults to None.
        disk_size(int, optional): The root device disk size (in GiB) for your node group instances. The default disk size is 20 GiB. If you specify launchTemplate, then don't specify diskSize, or the node group deployment will fail. For more information about using launch templates with Amazon EKS, see Launch template support in the Amazon EKS User Guide. Defaults to None.
        subnets(List): The subnets to use for the Auto Scaling group that is created for your node group. If you specify launchTemplate, then don't specify  SubnetId  in your launch template, or the node group deployment will fail. For more information about using launch templates with Amazon EKS, see Launch template support in the Amazon EKS User Guide.
        instance_types(List, optional): Specify the instance types for a node group. If you specify a GPU instance type, be sure to specify AL2_x86_64_GPU with the amiType parameter. If you specify launchTemplate, then you can specify zero or one instance type in your launch template or you can specify 0-20 instance types for instanceTypes. If however, you specify an instance type in your launch template and specify any instanceTypes, the node group deployment will fail. If you don't specify an instance type in a launch template or for instanceTypes, then t3.medium is used, by default. If you specify Spot for capacityType, then we recommend specifying multiple values for instanceTypes. For more information, see Managed node group capacity types and Launch template support in the Amazon EKS User Guide. Defaults to None.
        ami_type(str, optional): The AMI type for your node group. GPU instance types should use the AL2_x86_64_GPU AMI type. Non-GPU instances should use the AL2_x86_64 AMI type. Arm instances should use the AL2_ARM_64 AMI type. All types use the Amazon EKS optimized Amazon Linux 2 AMI. If you specify launchTemplate, and your launch template uses a custom AMI, then don't specify amiType, or the node group deployment will fail. For more information about using launch templates with Amazon EKS, see Launch template support in the Amazon EKS User Guide. Defaults to None.
        remote_access(Dict, optional): The remote access (SSH) configuration to use with your node group. If you specify launchTemplate, then don't specify remoteAccess, or the node group deployment will fail. For more information about using launch templates with Amazon EKS, see Launch template support in the Amazon EKS User Guide. Defaults to None.
        node_role(str): The Amazon Resource Name (ARN) of the IAM role to associate with your node group. The Amazon EKS worker node kubelet daemon makes calls to Amazon Web Services APIs on your behalf. Nodes receive permissions for these API calls through an IAM instance profile and associated policies. Before you can launch nodes and register them into a cluster, you must create an IAM role for those nodes to use when they are launched. For more information, see Amazon EKS node IAM role in the  Amazon EKS User Guide . If you specify launchTemplate, then don't specify  IamInstanceProfile  in your launch template, or the node group deployment will fail. For more information about using launch templates with Amazon EKS, see Launch template support in the Amazon EKS User Guide.
        labels(Dict, optional): The Kubernetes labels to be applied to the nodes in the node group when they are created. Defaults to None.
        taints(List, optional): The Kubernetes taints to be applied to the nodes in the node group. Defaults to None.
        tags(Dict, optional): The metadata to apply to the node group to assist with categorization and organization. Each tag consists of a key and an optional value, both of which you define. Node group tags do not propagate to any other resources associated with the node group, such as the Amazon EC2 instances or subnets. Defaults to None.
        client_request_token(str, optional): Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. Defaults to None.
        launch_template(Dict, optional): An object representing a node group's launch template specification. If specified, then do not specify instanceTypes, diskSize, or remoteAccess and make sure that the launch template meets the requirements in launchTemplateSpecification. Defaults to None.
        update_config(Dict, optional): The node group update configuration. Defaults to None.
        capacity_type(str, optional): The capacity type for your node group. Defaults to None.
        version(str, optional): The Kubernetes version to use for your managed nodes. By default, the Kubernetes version of the cluster is used, and this is the only accepted specified value. If you specify launchTemplate, and your launch template uses a custom AMI, then don't specify version, or the node group deployment will fail. For more information about using launch templates with Amazon EKS, see Launch template support in the Amazon EKS User Guide. Defaults to None.
        release_version(str, optional): The AMI version of the Amazon EKS optimized AMI to use with your node group. By default, the latest available AMI version for the node group's current Kubernetes version is used. For more information, see Amazon EKS optimized Amazon Linux 2 AMI versions in the Amazon EKS User Guide. If you specify launchTemplate, and your launch template uses a custom AMI, then don't specify releaseVersion, or the node group deployment will fail. For more information about using launch templates with Amazon EKS, see Launch template support in the Amazon EKS User Guide. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.eks.nodegroup.init.create(
                ctx, cluster_name=value, nodegroup_name=value, subnets=value, node_role=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.eks.nodegroup.init.create cluster_name=value, nodegroup_name=value, subnets=value, node_role=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="eks",
        operation="create_nodegroup",
        op_kwargs={
            "clusterName": cluster_name,
            "nodegroupName": nodegroup_name,
            "scalingConfig": scaling_config,
            "diskSize": disk_size,
            "subnets": subnets,
            "instanceTypes": instance_types,
            "amiType": ami_type,
            "remoteAccess": remote_access,
            "nodeRole": node_role,
            "labels": labels,
            "taints": taints,
            "tags": tags,
            "clientRequestToken": client_request_token,
            "launchTemplate": launch_template,
            "updateConfig": update_config,
            "capacityType": capacity_type,
            "version": version,
            "releaseVersion": release_version,
        },
    )


async def delete(hub, ctx, cluster_name: str, nodegroup_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes an Amazon EKS node group for a cluster.

    Args:
        cluster_name(str): The name of the Amazon EKS cluster that is associated with your node group.
        nodegroup_name(str): The name of the node group to delete.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.eks.nodegroup.init.delete(
                ctx, cluster_name=value, nodegroup_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.eks.nodegroup.init.delete cluster_name=value, nodegroup_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="eks",
        operation="delete_nodegroup",
        op_kwargs={"clusterName": cluster_name, "nodegroupName": nodegroup_name},
    )


async def describe(hub, ctx, cluster_name: str, nodegroup_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Returns descriptive information about an Amazon EKS node group.

    Args:
        cluster_name(str): The name of the Amazon EKS cluster associated with the node group.
        nodegroup_name(str): The name of the node group to describe.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.eks.nodegroup.init.describe(
                ctx, cluster_name=value, nodegroup_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.eks.nodegroup.init.describe cluster_name=value, nodegroup_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="eks",
        operation="describe_nodegroup",
        op_kwargs={"clusterName": cluster_name, "nodegroupName": nodegroup_name},
    )


async def list_all(
    hub, ctx, cluster_name: str, max_results: int = None, next_token: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Lists the Amazon EKS managed node groups associated with the specified cluster in your Amazon Web Services account in the specified Region. Self-managed node groups are not listed.

    Args:
        cluster_name(str): The name of the Amazon EKS cluster that you would like to list node groups in.
        max_results(int, optional): The maximum number of node group results returned by ListNodegroups in paginated output. When you use this parameter, ListNodegroups returns only maxResults results in a single page along with a nextToken response element. You can see the remaining results of the initial request by sending another ListNodegroups request with the returned nextToken value. This value can be between 1 and 100. If you don't use this parameter, ListNodegroups returns up to 100 results and a nextToken value if applicable. Defaults to None.
        next_token(str, optional): The nextToken value returned from a previous paginated ListNodegroups request where maxResults was used and the results exceeded the value of that parameter. Pagination continues from the end of the previous results that returned the nextToken value. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.eks.nodegroup.init.list_all(ctx, cluster_name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.eks.nodegroup.init.list_all cluster_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="eks",
        operation="list_nodegroups",
        op_kwargs={
            "clusterName": cluster_name,
            "maxResults": max_results,
            "nextToken": next_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import List


async def add(
    hub, ctx, queue_url: str, label: str, aws_account_ids: List, actions: List
) -> None:
    r"""
    **Autogenerated function**

    Adds a permission to a queue for a specific principal. This allows sharing access to the queue. When you create a queue, you have full control access rights for the queue. Only you, the owner of the queue, can grant or deny permissions to the queue. For more information about these permissions, see Allow Developers to Write Messages to a Shared Queue in the Amazon SQS Developer Guide.     AddPermission generates a policy for you. You can use  SetQueueAttributes  to upload your policy. For more information, see Using Custom Policies with the Amazon SQS Access Policy Language in the Amazon SQS Developer Guide.   An Amazon SQS policy can have a maximum of 7 actions.   To remove the ability to change queue permissions, you must deny permission to the AddPermission, RemovePermission, and SetQueueAttributes actions in your IAM policy.    Some actions take lists of parameters. These lists are specified using the param.n notation. Values of n are integers starting from 1. For example, a parameter list with two elements looks like this:  &AttributeName.1=first   &AttributeName.2=second   Cross-account permissions don't apply to this action. For more information, see Grant cross-account permissions to a role and a user name in the Amazon SQS Developer Guide.

    Args:
        queue_url(str): The URL of the Amazon SQS queue to which permissions are added. Queue URLs and names are case-sensitive.
        label(str): The unique identification of the permission you're setting (for example, AliceSendMessage). Maximum 80 characters. Allowed characters include alphanumeric characters, hyphens (-), and underscores (_).
        aws_account_ids(List): The account numbers of the principals who are to receive permission. For information about locating the account identification, see Your Amazon Web Services Identifiers in the Amazon SQS Developer Guide.
        actions(List): The action the client wants to allow for the specified principal. Valid values: the name of any action or *. For more information about these actions, see Overview of Managing Access Permissions to Your Amazon Simple Queue Service Resource in the Amazon SQS Developer Guide. Specifying SendMessage, DeleteMessage, or ChangeMessageVisibility for ActionName.n also grants permissions for the corresponding batch versions of those actions: SendMessageBatch, DeleteMessageBatch, and ChangeMessageVisibilityBatch.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sqs.permission.add(
                ctx, queue_url=value, label=value, aws_account_ids=value, actions=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sqs.permission.add queue_url=value, label=value, aws_account_ids=value, actions=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sqs",
        operation="add_permission",
        op_kwargs={
            "QueueUrl": queue_url,
            "Label": label,
            "AWSAccountIds": aws_account_ids,
            "Actions": actions,
        },
    )


async def remove(hub, ctx, queue_url: str, label: str) -> None:
    r"""
    **Autogenerated function**

    Revokes any permissions in the queue policy that matches the specified Label parameter.    Only the owner of a queue can remove permissions from it.   Cross-account permissions don't apply to this action. For more information, see Grant cross-account permissions to a role and a user name in the Amazon SQS Developer Guide.   To remove the ability to change queue permissions, you must deny permission to the AddPermission, RemovePermission, and SetQueueAttributes actions in your IAM policy.

    Args:
        queue_url(str): The URL of the Amazon SQS queue from which permissions are removed. Queue URLs and names are case-sensitive.
        label(str): The identification of the permission to remove. This is the label added using the  AddPermission  action.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sqs.permission.remove(ctx, queue_url=value, label=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sqs.permission.remove queue_url=value, label=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sqs",
        operation="remove_permission",
        op_kwargs={"QueueUrl": queue_url, "Label": label},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub, ctx, end_points: List, fields: List, name: str, sampling_rate: int
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a real-time log configuration. After you create a real-time log configuration, you can attach it to one or more cache behaviors to send real-time log data to the specified Amazon Kinesis data stream. For more information about real-time log configurations, see Real-time logs in the Amazon CloudFront Developer Guide.

    Args:
        end_points(List): Contains information about the Amazon Kinesis data stream where you are sending real-time log data.
        fields(List): A list of fields to include in each real-time log record. For more information about fields, see Real-time log configuration fields in the Amazon CloudFront Developer Guide.
        name(str): A unique name to identify this real-time log configuration.
        sampling_rate(int): The sampling rate for this real-time log configuration. The sampling rate determines the percentage of viewer requests that are represented in the real-time log data. You must provide an integer between 1 and 100, inclusive.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.realtime_log_config.create(
                ctx, end_points=value, fields=value, name=value, sampling_rate=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.realtime_log_config.create end_points=value, fields=value, name=value, sampling_rate=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="create_realtime_log_config",
        op_kwargs={
            "EndPoints": end_points,
            "Fields": fields,
            "Name": name,
            "SamplingRate": sampling_rate,
        },
    )


async def delete(hub, ctx, name: str = None, arn: str = None) -> None:
    r"""
    **Autogenerated function**

    Deletes a real-time log configuration. You cannot delete a real-time log configuration if it’s attached to a cache behavior. First update your distributions to remove the real-time log configuration from all cache behaviors, then delete the real-time log configuration. To delete a real-time log configuration, you can provide the configuration’s name or its Amazon Resource Name (ARN). You must provide at least one. If you provide both, CloudFront uses the name to identify the real-time log configuration to delete.

    Args:
        name(str, optional): The name of the real-time log configuration to delete. Defaults to None.
        arn(str, optional): The Amazon Resource Name (ARN) of the real-time log configuration to delete. Defaults to None.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.realtime_log_config.delete(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.realtime_log_config.delete
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="delete_realtime_log_config",
        op_kwargs={"Name": name, "ARN": arn},
    )


async def get(hub, ctx, name: str = None, arn: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Gets a real-time log configuration. To get a real-time log configuration, you can provide the configuration’s name or its Amazon Resource Name (ARN). You must provide at least one. If you provide both, CloudFront uses the name to identify the real-time log configuration to get.

    Args:
        name(str, optional): The name of the real-time log configuration to get. Defaults to None.
        arn(str, optional): The Amazon Resource Name (ARN) of the real-time log configuration to get. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.realtime_log_config.get(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.realtime_log_config.get
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="get_realtime_log_config",
        op_kwargs={"Name": name, "ARN": arn},
    )


async def list_all(hub, ctx, max_items: str = None, marker: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Gets a list of real-time log configurations. You can optionally specify the maximum number of items to receive in the response. If the total number of items in the list exceeds the maximum that you specify, or the default maximum, the response is paginated. To get the next page of items, send a subsequent request that specifies the NextMarker value from the current response as the Marker value in the subsequent request.

    Args:
        max_items(str, optional): The maximum number of real-time log configurations that you want in the response. Defaults to None.
        marker(str, optional): Use this field when paginating results to indicate where to begin in your list of real-time log configurations. The response includes real-time log configurations in the list that occur after the marker. To get the next page of the list, set this field’s value to the value of NextMarker from the current page’s response. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.realtime_log_config.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.realtime_log_config.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="list_realtime_log_configs",
        op_kwargs={"MaxItems": max_items, "Marker": marker},
    )


async def update(
    hub,
    ctx,
    end_points: List = None,
    fields: List = None,
    name: str = None,
    arn: str = None,
    sampling_rate: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Updates a real-time log configuration. When you update a real-time log configuration, all the parameters are updated with the values provided in the request. You cannot update some parameters independent of others. To update a real-time log configuration:   Call GetRealtimeLogConfig to get the current real-time log configuration.   Locally modify the parameters in the real-time log configuration that you want to update.   Call this API (UpdateRealtimeLogConfig) by providing the entire real-time log configuration, including the parameters that you modified and those that you didn’t.   You cannot update a real-time log configuration’s Name or ARN.

    Args:
        end_points(List, optional): Contains information about the Amazon Kinesis data stream where you are sending real-time log data. Defaults to None.
        fields(List, optional): A list of fields to include in each real-time log record. For more information about fields, see Real-time log configuration fields in the Amazon CloudFront Developer Guide. Defaults to None.
        name(str, optional): The name for this real-time log configuration. Defaults to None.
        arn(str, optional): The Amazon Resource Name (ARN) for this real-time log configuration. Defaults to None.
        sampling_rate(int, optional): The sampling rate for this real-time log configuration. The sampling rate determines the percentage of viewer requests that are represented in the real-time log data. You must provide an integer between 1 and 100, inclusive. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.realtime_log_config.update(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.realtime_log_config.update
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="update_realtime_log_config",
        op_kwargs={
            "EndPoints": end_points,
            "Fields": fields,
            "Name": name,
            "ARN": arn,
            "SamplingRate": sampling_rate,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(
    hub, ctx, data_set_id: str, comment: str = None, tags: Dict = None
) -> Dict:
    r"""
    **Autogenerated function**

    This operation creates a revision for a data set.

    Args:
        comment(str, optional): An optional comment about the revision. Defaults to None.
        data_set_id(str): The unique identifier for a data set.
        tags(Dict, optional): A revision tag is an optional label that you can assign to a revision when you create it. Each tag consists of a key and an optional value, both of which you define. When you use tagging, you can also use tag-based access control in IAM policies to control access to these data sets and revisions. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dataexchange.revision.init.create(ctx, data_set_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dataexchange.revision.init.create data_set_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dataexchange",
        operation="create_revision",
        op_kwargs={"Comment": comment, "DataSetId": data_set_id, "Tags": tags},
    )


async def delete(hub, ctx, data_set_id: str, revision_id: str) -> None:
    r"""
    **Autogenerated function**

    This operation deletes a revision.

    Args:
        data_set_id(str): The unique identifier for a data set.
        revision_id(str): The unique identifier for a revision.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dataexchange.revision.init.delete(
                ctx, data_set_id=value, revision_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dataexchange.revision.init.delete data_set_id=value, revision_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dataexchange",
        operation="delete_revision",
        op_kwargs={"DataSetId": data_set_id, "RevisionId": revision_id},
    )


async def get(hub, ctx, data_set_id: str, revision_id: str) -> Dict:
    r"""
    **Autogenerated function**

    This operation returns information about a revision.

    Args:
        data_set_id(str): The unique identifier for a data set.
        revision_id(str): The unique identifier for a revision.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dataexchange.revision.init.get(
                ctx, data_set_id=value, revision_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dataexchange.revision.init.get data_set_id=value, revision_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dataexchange",
        operation="get_revision",
        op_kwargs={"DataSetId": data_set_id, "RevisionId": revision_id},
    )


async def update(
    hub,
    ctx,
    data_set_id: str,
    revision_id: str,
    comment: str = None,
    finalized: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    This operation updates a revision.

    Args:
        comment(str, optional): An optional comment about the revision. Defaults to None.
        data_set_id(str): The unique identifier for a data set.
        finalized(bool, optional): Finalizing a revision tells AWS Data Exchange that your changes to the assets in the revision are complete. After it's in this read-only state, you can publish the revision to your products. Defaults to None.
        revision_id(str): The unique identifier for a revision.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.dataexchange.revision.init.update(
                ctx, data_set_id=value, revision_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.dataexchange.revision.init.update data_set_id=value, revision_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="dataexchange",
        operation="update_revision",
        op_kwargs={
            "Comment": comment,
            "DataSetId": data_set_id,
            "Finalized": finalized,
            "RevisionId": revision_id,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def list_all(
    hub,
    ctx,
    user_name: str,
    aws_account_id: str,
    namespace: str,
    next_token: str = None,
    max_results: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Lists the Amazon QuickSight groups that an Amazon QuickSight user is a member of.

    Args:
        user_name(str): The Amazon QuickSight user name that you want to list group memberships for.
        aws_account_id(str): The Amazon Web Services account; ID that the user is in. Currently, you use the ID for the Amazon Web Services account; that contains your Amazon QuickSight account.
        namespace(str): The namespace. Currently, you should set this to default.
        next_token(str, optional): A pagination token that can be used in a subsequent request. Defaults to None.
        max_results(int, optional): The maximum number of results to return from this request. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.quicksight.user.group.list_all(
                ctx, user_name=value, aws_account_id=value, namespace=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.quicksight.user.group.list_all user_name=value, aws_account_id=value, namespace=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="quicksight",
        operation="list_user_groups",
        op_kwargs={
            "UserName": user_name,
            "AwsAccountId": aws_account_id,
            "Namespace": namespace,
            "NextToken": next_token,
            "MaxResults": max_results,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def get(hub, ctx, pipeline_id: str, version: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Gets the definition of the specified pipeline. You can call GetPipelineDefinition to retrieve the pipeline definition that you provided using PutPipelineDefinition.

    Args:
        pipeline_id(str): The ID of the pipeline.
        version(str, optional): The version of the pipeline definition to retrieve. Set this parameter to latest (default) to use the last definition saved to the pipeline or active to use the last definition that was activated. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.datapipeline.pipeline.definition.get(ctx, pipeline_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.datapipeline.pipeline.definition.get pipeline_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="datapipeline",
        operation="get_pipeline_definition",
        op_kwargs={"pipelineId": pipeline_id, "version": version},
    )


async def put(
    hub,
    ctx,
    pipeline_id: str,
    pipeline_objects: List,
    parameter_objects: List = None,
    parameter_values: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Adds tasks, schedules, and preconditions to the specified pipeline. You can use PutPipelineDefinition to populate a new pipeline.  PutPipelineDefinition also validates the configuration as it adds it to the pipeline. Changes to the pipeline are saved unless one of the following three validation errors exists in the pipeline.   An object is missing a name or identifier field. A string or reference field is empty. The number of objects in the pipeline exceeds the maximum allowed objects. The pipeline is in a FINISHED state.   Pipeline object definitions are passed to the PutPipelineDefinition action and returned by the GetPipelineDefinition action.

    Args:
        pipeline_id(str): The ID of the pipeline.
        pipeline_objects(List): The objects that define the pipeline. These objects overwrite the existing pipeline definition.
        parameter_objects(List, optional): The parameter objects used with the pipeline. Defaults to None.
        parameter_values(List, optional): The parameter values used with the pipeline. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.datapipeline.pipeline.definition.put(
                ctx, pipeline_id=value, pipeline_objects=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.datapipeline.pipeline.definition.put pipeline_id=value, pipeline_objects=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="datapipeline",
        operation="put_pipeline_definition",
        op_kwargs={
            "pipelineId": pipeline_id,
            "pipelineObjects": pipeline_objects,
            "parameterObjects": parameter_objects,
            "parameterValues": parameter_values,
        },
    )


async def validate(
    hub,
    ctx,
    pipeline_id: str,
    pipeline_objects: List,
    parameter_objects: List = None,
    parameter_values: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Validates the specified pipeline definition to ensure that it is well formed and can be run without error.

    Args:
        pipeline_id(str): The ID of the pipeline.
        pipeline_objects(List): The objects that define the pipeline changes to validate against the pipeline.
        parameter_objects(List, optional): The parameter objects used with the pipeline. Defaults to None.
        parameter_values(List, optional): The parameter values used with the pipeline. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.datapipeline.pipeline.definition.validate(
                ctx, pipeline_id=value, pipeline_objects=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.datapipeline.pipeline.definition.validate pipeline_id=value, pipeline_objects=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="datapipeline",
        operation="validate_pipeline_definition",
        op_kwargs={
            "pipelineId": pipeline_id,
            "pipelineObjects": pipeline_objects,
            "parameterObjects": parameter_objects,
            "parameterValues": parameter_values,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get(hub, ctx) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves the current encryption configuration for X-Ray data.


    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.xray.encryption_config.get(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.xray.encryption_config.get
    """
    return await hub.tool.aws.client.exec(
        ctx, service_name="xray", operation="get_encryption_config", op_kwargs={}
    )


async def put(hub, ctx, type_: str, key_id: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Updates the encryption configuration for X-Ray data.

    Args:
        key_id(str, optional): An AWS KMS customer master key (CMK) in one of the following formats:    Alias - The name of the key. For example, alias/MyKey.    Key ID - The KMS key ID of the key. For example, ae4aa6d49-a4d8-9df9-a475-4ff6d7898456. AWS X-Ray does not support asymmetric CMKs.    ARN - The full Amazon Resource Name of the key ID or alias. For example, arn:aws:kms:us-east-2:123456789012:key/ae4aa6d49-a4d8-9df9-a475-4ff6d7898456. Use this format to specify a key in a different account.   Omit this key if you set Type to NONE. Defaults to None.
        type_(str): The type of encryption. Set to KMS to use your own key for encryption. Set to NONE for default encryption.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.xray.encryption_config.put(ctx, type_=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.xray.encryption_config.put type_=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="xray",
        operation="put_encryption_config",
        op_kwargs={"KeyId": key_id, "Type": type_},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(hub, ctx, name: str, tags: Dict = None) -> Dict:
    r"""
    **Autogenerated function**

    Creates a fleet, a logical group of robots running the same robot application.

    Args:
        name(str): The name of the fleet.
        tags(Dict, optional): A map that contains tag keys and tag values that are attached to the fleet. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.robomaker.fleet.create(ctx, name=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.robomaker.fleet.create name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="robomaker",
        operation="create_fleet",
        op_kwargs={"name": name, "tags": tags},
    )


async def delete(hub, ctx, fleet: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes a fleet.

    Args:
        fleet(str): The Amazon Resource Name (ARN) of the fleet.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.robomaker.fleet.delete(ctx, fleet=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.robomaker.fleet.delete fleet=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="robomaker",
        operation="delete_fleet",
        op_kwargs={"fleet": fleet},
    )


async def describe(hub, ctx, fleet: str) -> Dict:
    r"""
    **Autogenerated function**

    Describes a fleet.

    Args:
        fleet(str): The Amazon Resource Name (ARN) of the fleet.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.robomaker.fleet.describe(ctx, fleet=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.robomaker.fleet.describe fleet=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="robomaker",
        operation="describe_fleet",
        op_kwargs={"fleet": fleet},
    )


async def list_all(
    hub, ctx, next_token: str = None, max_results: int = None, filters: List = None
) -> Dict:
    r"""
    **Autogenerated function**

    Returns a list of fleets. You can optionally provide filters to retrieve specific fleets.

    Args:
        next_token(str, optional): If the previous paginated request did not return all of the remaining results, the response object's nextToken parameter value is set to a token. To retrieve the next set of results, call ListFleets again and assign that token to the request object's nextToken parameter. If there are no remaining results, the previous response object's NextToken parameter is set to null.   This token should be treated as an opaque identifier that is only used to retrieve the next items in a list and not for other programmatic purposes. . Defaults to None.
        max_results(int, optional): When this parameter is used, ListFleets only returns maxResults results in a single page along with a nextToken response element. The remaining results of the initial request can be seen by sending another ListFleets request with the returned nextToken value. This value can be between 1 and 200. If this parameter is not used, then ListFleets returns up to 200 results and a nextToken value if applicable. . Defaults to None.
        filters(List, optional): Optional filters to limit results. The filter name name is supported. When filtering, you must use the complete value of the filtered item. You can use up to three filters. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.robomaker.fleet.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.robomaker.fleet.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="robomaker",
        operation="list_fleets",
        op_kwargs={
            "nextToken": next_token,
            "maxResults": max_results,
            "filters": filters,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def execute(
    hub,
    ctx,
    resource_arn: str,
    secret_arn: str,
    sql: str,
    continue_after_timeout: bool = None,
    database: str = None,
    include_result_metadata: bool = None,
    parameters: List = None,
    result_set_options: Dict = None,
    schema: str = None,
    transaction_id: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Runs a SQL statement against a database.  If a call isn't part of a transaction because it doesn't include the transactionID parameter, changes that result from the call are committed automatically.  The response size limit is 1 MB. If the call returns more than 1 MB of response data, the call is terminated.

    Args:
        continue_after_timeout(bool, optional): A value that indicates whether to continue running the statement after the call times out. By default, the statement stops running when the call times out.  For DDL statements, we recommend continuing to run the statement after the call times out. When a DDL statement terminates before it is finished running, it can result in errors and possibly corrupted data structures. . Defaults to None.
        database(str, optional): The name of the database. Defaults to None.
        include_result_metadata(bool, optional): A value that indicates whether to include metadata in the results. Defaults to None.
        parameters(List, optional): The parameters for the SQL statement.  Array parameters are not supported. . Defaults to None.
        resource_arn(str): The Amazon Resource Name (ARN) of the Aurora Serverless DB cluster.
        result_set_options(Dict, optional): Options that control how the result set is returned. Defaults to None.
        schema(str, optional): The name of the database schema.  Currently, the schema parameter isn't supported. . Defaults to None.
        secret_arn(str): The name or ARN of the secret that enables access to the DB cluster.
        sql(str): The SQL statement to run.
        transaction_id(str, optional): The identifier of a transaction that was started by using the BeginTransaction operation. Specify the transaction ID of the transaction that you want to include the SQL statement in. If the SQL statement is not part of a transaction, don't set this parameter. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.rds_data.statement.execute(
                ctx, resource_arn=value, secret_arn=value, sql=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.rds_data.statement.execute resource_arn=value, secret_arn=value, sql=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="rds-data",
        operation="execute_statement",
        op_kwargs={
            "continueAfterTimeout": continue_after_timeout,
            "database": database,
            "includeResultMetadata": include_result_metadata,
            "parameters": parameters,
            "resourceArn": resource_arn,
            "resultSetOptions": result_set_options,
            "schema": schema,
            "secretArn": secret_arn,
            "sql": sql,
            "transactionId": transaction_id,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    certificate_authority_configuration: Dict,
    certificate_authority_type: str,
    revocation_configuration: Dict = None,
    idempotency_token: str = None,
    key_storage_security_standard: str = None,
    tags: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a root or subordinate private certificate authority (CA). You must specify the CA configuration, the certificate revocation list (CRL) configuration, the CA type, and an optional idempotency token to avoid accidental creation of multiple CAs. The CA configuration specifies the name of the algorithm and key size to be used to create the CA private key, the type of signing algorithm that the CA uses, and X.500 subject information. The CRL configuration specifies the CRL expiration period in days (the validity period of the CRL), the Amazon S3 bucket that will contain the CRL, and a CNAME alias for the S3 bucket that is included in certificates issued by the CA. If successful, this action returns the Amazon Resource Name (ARN) of the CA. ACM Private CA assets that are stored in Amazon S3 can be protected with encryption. For more information, see Encrypting Your CRLs.  Both PCA and the IAM principal must have permission to write to the S3 bucket that you specify. If the IAM principal making the call does not have permission to write to the bucket, then an exception is thrown. For more information, see Configure Access to ACM Private CA.

    Args:
        certificate_authority_configuration(Dict): Name and bit size of the private key algorithm, the name of the signing algorithm, and X.500 certificate subject information.
        revocation_configuration(Dict, optional): Contains a Boolean value that you can use to enable a certification revocation list (CRL) for the CA, the name of the S3 bucket to which ACM Private CA will write the CRL, and an optional CNAME alias that you can use to hide the name of your bucket in the CRL Distribution Points extension of your CA certificate. For more information, see the CrlConfiguration structure. . Defaults to None.
        certificate_authority_type(str): The type of the certificate authority.
        idempotency_token(str, optional): Custom string that can be used to distinguish between calls to the CreateCertificateAuthority action. Idempotency tokens for CreateCertificateAuthority time out after five minutes. Therefore, if you call CreateCertificateAuthority multiple times with the same idempotency token within five minutes, ACM Private CA recognizes that you are requesting only certificate authority and will issue only one. If you change the idempotency token for each call, PCA recognizes that you are requesting multiple certificate authorities. Defaults to None.
        key_storage_security_standard(str, optional): Specifies a cryptographic key management compliance standard used for handling CA keys. Default: FIPS_140_2_LEVEL_3_OR_HIGHER Note: FIPS_140_2_LEVEL_3_OR_HIGHER is not supported in Region ap-northeast-3. When creating a CA in the ap-northeast-3, you must provide FIPS_140_2_LEVEL_2_OR_HIGHER as the argument for KeyStorageSecurityStandard. Failure to do this results in an InvalidArgsException with the message, "A certificate authority cannot be created in this region with the specified security standard.". Defaults to None.
        tags(List, optional): Key-value pairs that will be attached to the new private CA. You can associate up to 50 tags with a private CA. For information using tags with IAM to manage permissions, see Controlling Access Using IAM Tags. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.acm_pca.certificate.authority.init.create(
                ctx, certificate_authority_configuration=value, certificate_authority_type=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.acm_pca.certificate.authority.init.create certificate_authority_configuration=value, certificate_authority_type=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="acm-pca",
        operation="create_certificate_authority",
        op_kwargs={
            "CertificateAuthorityConfiguration": certificate_authority_configuration,
            "RevocationConfiguration": revocation_configuration,
            "CertificateAuthorityType": certificate_authority_type,
            "IdempotencyToken": idempotency_token,
            "KeyStorageSecurityStandard": key_storage_security_standard,
            "Tags": tags,
        },
    )


async def delete(
    hub,
    ctx,
    certificate_authority_arn: str,
    permanent_deletion_time_in_days: int = None,
) -> None:
    r"""
    **Autogenerated function**

    Deletes a private certificate authority (CA). You must provide the Amazon Resource Name (ARN) of the private CA that you want to delete. You can find the ARN by calling the ListCertificateAuthorities action.   Deleting a CA will invalidate other CAs and certificates below it in your CA hierarchy.  Before you can delete a CA that you have created and activated, you must disable it. To do this, call the UpdateCertificateAuthority action and set the CertificateAuthorityStatus parameter to DISABLED.  Additionally, you can delete a CA if you are waiting for it to be created (that is, the status of the CA is CREATING). You can also delete it if the CA has been created but you haven't yet imported the signed certificate into ACM Private CA (that is, the status of the CA is PENDING_CERTIFICATE).  When you successfully call DeleteCertificateAuthority, the CA's status changes to DELETED. However, the CA won't be permanently deleted until the restoration period has passed. By default, if you do not set the PermanentDeletionTimeInDays parameter, the CA remains restorable for 30 days. You can set the parameter from 7 to 30 days. The DescribeCertificateAuthority action returns the time remaining in the restoration window of a private CA in the DELETED state. To restore an eligible CA, call the RestoreCertificateAuthority action.

    Args:
        certificate_authority_arn(str): The Amazon Resource Name (ARN) that was returned when you called CreateCertificateAuthority. This must have the following form:   arn:aws:acm-pca:region:account:certificate-authority/12345678-1234-1234-1234-123456789012 . .
        permanent_deletion_time_in_days(int, optional): The number of days to make a CA restorable after it has been deleted. This can be anywhere from 7 to 30 days, with 30 being the default. Defaults to None.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.acm_pca.certificate.authority.init.delete(
                ctx, certificate_authority_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.acm_pca.certificate.authority.init.delete certificate_authority_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="acm-pca",
        operation="delete_certificate_authority",
        op_kwargs={
            "CertificateAuthorityArn": certificate_authority_arn,
            "PermanentDeletionTimeInDays": permanent_deletion_time_in_days,
        },
    )


async def describe(hub, ctx, certificate_authority_arn: str) -> Dict:
    r"""
    **Autogenerated function**

    Lists information about your private certificate authority (CA) or one that has been shared with you. You specify the private CA on input by its ARN (Amazon Resource Name). The output contains the status of your CA. This can be any of the following:     CREATING - ACM Private CA is creating your private certificate authority.    PENDING_CERTIFICATE - The certificate is pending. You must use your ACM Private CA-hosted or on-premises root or subordinate CA to sign your private CA CSR and then import it into PCA.     ACTIVE - Your private CA is active.    DISABLED - Your private CA has been disabled.    EXPIRED - Your private CA certificate has expired.    FAILED - Your private CA has failed. Your CA can fail because of problems such a network outage or back-end AWS failure or other errors. A failed CA can never return to the pending state. You must create a new CA.     DELETED - Your private CA is within the restoration period, after which it is permanently deleted. The length of time remaining in the CA's restoration period is also included in this action's output.

    Args:
        certificate_authority_arn(str): The Amazon Resource Name (ARN) that was returned when you called CreateCertificateAuthority. This must be of the form:   arn:aws:acm-pca:region:account:certificate-authority/12345678-1234-1234-1234-123456789012 . .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.acm_pca.certificate.authority.init.describe(
                ctx, certificate_authority_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.acm_pca.certificate.authority.init.describe certificate_authority_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="acm-pca",
        operation="describe_certificate_authority",
        op_kwargs={"CertificateAuthorityArn": certificate_authority_arn},
    )


async def list_all(
    hub,
    ctx,
    next_token: str = None,
    max_results: int = None,
    resource_owner: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Lists the private certificate authorities that you created by using the CreateCertificateAuthority action.

    Args:
        next_token(str, optional): Use this parameter when paginating results in a subsequent request after you receive a response with truncated results. Set it to the value of the NextToken parameter from the response you just received. Defaults to None.
        max_results(int, optional): Use this parameter when paginating results to specify the maximum number of items to return in the response on each page. If additional items exist beyond the number you specify, the NextToken element is sent in the response. Use this NextToken value in a subsequent request to retrieve additional items. Defaults to None.
        resource_owner(str, optional): Use this parameter to filter the returned set of certificate authorities based on their owner. The default is SELF. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.acm_pca.certificate.authority.init.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.acm_pca.certificate.authority.init.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="acm-pca",
        operation="list_certificate_authorities",
        op_kwargs={
            "NextToken": next_token,
            "MaxResults": max_results,
            "ResourceOwner": resource_owner,
        },
    )


async def restore(hub, ctx, certificate_authority_arn: str) -> None:
    r"""
    **Autogenerated function**

    Restores a certificate authority (CA) that is in the DELETED state. You can restore a CA during the period that you defined in the PermanentDeletionTimeInDays parameter of the DeleteCertificateAuthority action. Currently, you can specify 7 to 30 days. If you did not specify a PermanentDeletionTimeInDays value, by default you can restore the CA at any time in a 30 day period. You can check the time remaining in the restoration period of a private CA in the DELETED state by calling the DescribeCertificateAuthority or ListCertificateAuthorities actions. The status of a restored CA is set to its pre-deletion status when the RestoreCertificateAuthority action returns. To change its status to ACTIVE, call the UpdateCertificateAuthority action. If the private CA was in the PENDING_CERTIFICATE state at deletion, you must use the ImportCertificateAuthorityCertificate action to import a certificate authority into the private CA before it can be activated. You cannot restore a CA after the restoration period has ended.

    Args:
        certificate_authority_arn(str): The Amazon Resource Name (ARN) that was returned when you called the CreateCertificateAuthority action. This must be of the form:   arn:aws:acm-pca:region:account:certificate-authority/12345678-1234-1234-1234-123456789012  .
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.acm_pca.certificate.authority.init.restore(
                ctx, certificate_authority_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.acm_pca.certificate.authority.init.restore certificate_authority_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="acm-pca",
        operation="restore_certificate_authority",
        op_kwargs={"CertificateAuthorityArn": certificate_authority_arn},
    )


async def tag(hub, ctx, certificate_authority_arn: str, tags: List) -> None:
    r"""
    **Autogenerated function**

    Adds one or more tags to your private CA. Tags are labels that you can use to identify and organize your AWS resources. Each tag consists of a key and an optional value. You specify the private CA on input by its Amazon Resource Name (ARN). You specify the tag by using a key-value pair. You can apply a tag to just one private CA if you want to identify a specific characteristic of that CA, or you can apply the same tag to multiple private CAs if you want to filter for a common relationship among those CAs. To remove one or more tags, use the UntagCertificateAuthority action. Call the ListTags action to see what tags are associated with your CA.

    Args:
        certificate_authority_arn(str): The Amazon Resource Name (ARN) that was returned when you called CreateCertificateAuthority. This must be of the form:   arn:aws:acm-pca:region:account:certificate-authority/12345678-1234-1234-1234-123456789012  .
        tags(List): List of tags to be associated with the CA.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.acm_pca.certificate.authority.init.tag(
                ctx, certificate_authority_arn=value, tags=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.acm_pca.certificate.authority.init.tag certificate_authority_arn=value, tags=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="acm-pca",
        operation="tag_certificate_authority",
        op_kwargs={"CertificateAuthorityArn": certificate_authority_arn, "Tags": tags},
    )


async def untag(hub, ctx, certificate_authority_arn: str, tags: List) -> None:
    r"""
    **Autogenerated function**

    Remove one or more tags from your private CA. A tag consists of a key-value pair. If you do not specify the value portion of the tag when calling this action, the tag will be removed regardless of value. If you specify a value, the tag is removed only if it is associated with the specified value. To add tags to a private CA, use the TagCertificateAuthority. Call the ListTags action to see what tags are associated with your CA.

    Args:
        certificate_authority_arn(str): The Amazon Resource Name (ARN) that was returned when you called CreateCertificateAuthority. This must be of the form:   arn:aws:acm-pca:region:account:certificate-authority/12345678-1234-1234-1234-123456789012  .
        tags(List): List of tags to be removed from the CA.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.acm_pca.certificate.authority.init.untag(
                ctx, certificate_authority_arn=value, tags=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.acm_pca.certificate.authority.init.untag certificate_authority_arn=value, tags=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="acm-pca",
        operation="untag_certificate_authority",
        op_kwargs={"CertificateAuthorityArn": certificate_authority_arn, "Tags": tags},
    )


async def update(
    hub,
    ctx,
    certificate_authority_arn: str,
    revocation_configuration: Dict = None,
    status: str = None,
) -> None:
    r"""
    **Autogenerated function**

    Updates the status or configuration of a private certificate authority (CA). Your private CA must be in the ACTIVE or DISABLED state before you can update it. You can disable a private CA that is in the ACTIVE state or make a CA that is in the DISABLED state active again.  Both PCA and the IAM principal must have permission to write to the S3 bucket that you specify. If the IAM principal making the call does not have permission to write to the bucket, then an exception is thrown. For more information, see Configure Access to ACM Private CA.

    Args:
        certificate_authority_arn(str): Amazon Resource Name (ARN) of the private CA that issued the certificate to be revoked. This must be of the form:  arn:aws:acm-pca:region:account:certificate-authority/12345678-1234-1234-1234-123456789012  .
        revocation_configuration(Dict, optional): Revocation information for your private CA. Defaults to None.
        status(str, optional): Status of your private CA. Defaults to None.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.acm_pca.certificate.authority.init.update(
                ctx, certificate_authority_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.acm_pca.certificate.authority.init.update certificate_authority_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="acm-pca",
        operation="update_certificate_authority",
        op_kwargs={
            "CertificateAuthorityArn": certificate_authority_arn,
            "RevocationConfiguration": revocation_configuration,
            "Status": status,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def authorize(
    hub,
    ctx,
    snapshot_identifier: str,
    account_with_restore_access: str,
    snapshot_cluster_identifier: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Authorizes the specified Amazon Web Services account to restore the specified snapshot.  For more information about working with snapshots, go to Amazon Redshift Snapshots in the Amazon Redshift Cluster Management Guide.

    Args:
        snapshot_identifier(str): The identifier of the snapshot the account is authorized to restore.
        snapshot_cluster_identifier(str, optional): The identifier of the cluster the snapshot was created from. This parameter is required if your IAM user has a policy containing a snapshot resource element that specifies anything other than * for the cluster name. Defaults to None.
        account_with_restore_access(str): The identifier of the Amazon Web Services account authorized to restore the specified snapshot. To share a snapshot with Amazon Web Services Support, specify amazon-redshift-support.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.redshift.snapshot.access.authorize(
                ctx, snapshot_identifier=value, account_with_restore_access=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.redshift.snapshot.access.authorize snapshot_identifier=value, account_with_restore_access=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="redshift",
        operation="authorize_snapshot_access",
        op_kwargs={
            "SnapshotIdentifier": snapshot_identifier,
            "SnapshotClusterIdentifier": snapshot_cluster_identifier,
            "AccountWithRestoreAccess": account_with_restore_access,
        },
    )


async def revoke(
    hub,
    ctx,
    snapshot_identifier: str,
    account_with_restore_access: str,
    snapshot_cluster_identifier: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Removes the ability of the specified Amazon Web Services account to restore the specified snapshot. If the account is currently restoring the snapshot, the restore will run to completion.  For more information about working with snapshots, go to Amazon Redshift Snapshots in the Amazon Redshift Cluster Management Guide.

    Args:
        snapshot_identifier(str): The identifier of the snapshot that the account can no longer access.
        snapshot_cluster_identifier(str, optional): The identifier of the cluster the snapshot was created from. This parameter is required if your IAM user has a policy containing a snapshot resource element that specifies anything other than * for the cluster name. Defaults to None.
        account_with_restore_access(str): The identifier of the Amazon Web Services account that can no longer restore the specified snapshot.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.redshift.snapshot.access.revoke(
                ctx, snapshot_identifier=value, account_with_restore_access=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.redshift.snapshot.access.revoke snapshot_identifier=value, account_with_restore_access=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="redshift",
        operation="revoke_snapshot_access",
        op_kwargs={
            "SnapshotIdentifier": snapshot_identifier,
            "SnapshotClusterIdentifier": snapshot_cluster_identifier,
            "AccountWithRestoreAccess": account_with_restore_access,
        },
    )
